# Generated by Django 2.1.8 on 2019-04-02 15:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('for_runners', '0007_auto_20190402_1628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gpxmodel',
            name='participation',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='track', to='for_runners.ParticipationModel'),
        ),
        migrations.AlterField(
            model_name='gpxmodel',
            name='tracked_by',
            field=models.ForeignKey(default=1, editable=False, help_text='The user that tracked this gpx entry', on_delete=django.db.models.deletion.PROTECT, related_name='gpxmodel_createby', to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
    ]



"""
    Test API connection to metaweather.com via for_runners.weather

    https://www.metaweather.com/api/

    created 21.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import datetime

import requests_mock

from for_runners.tests.base import BaseTestCase
from for_runners.tests.fixture_files import fixture_content
from for_runners.tests.utils import ClearCacheMixin
from for_runners.weather import NoWeatherData, meta_weather_com


class WeatherTest(ClearCacheMixin, BaseTestCase):
    def test(self):
        lat, lon = (51.4109, 6.7828)  # Duisburg -> WOEID: 648820 (Essen, city)
        date = datetime.datetime(year=2018, month=6, day=20, hour=20, minute=30)

        with requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.41,6.78',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5141_678.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/6/20/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_6_20.json')
            )
            temperature, weather_state = meta_weather_com.coordinates2weather(
                lat, lon, date=date, max_seconds=12 * 60 * 60
            )

            self.assert_equal_rounded(temperature, 25.41, decimal_places=2)
            self.assertEqual(weather_state, "Light Cloud/Showers")

    def test_small_max_seconds(self):
        lat, lon = (51.4109, 6.7828)  # Duisburg -> WOEID: 648820 (Essen, city)
        date = datetime.datetime(year=2018, month=6, day=20, hour=20, minute=30)

        with requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.41,6.78',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5141_678.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/6/20/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_6_20.json')
            )
            temperature, weather_state = meta_weather_com.coordinates2weather(
                lat, lon, date=date, max_seconds=0.1
            )

            self.assert_equal_rounded(temperature, 27.94, decimal_places=2)
            self.assertEqual(weather_state, "Light Cloud")

    def test_no_json_data(self):
        """
        Request weather for Essen City on 11.02.2017 will raise into 500 - Server error:
            https://www.metaweather.com/de/648820/2017/2/11/

        The API Request will return []:
            https://www.metaweather.com/api/location/648820/2017/2/11/

        MetaWeatherCom().location_day() will raise NoWeatherData
        """
        lat, lon = (51.4109, 6.7828)  # Duisburg -> WOEID: 648820 (Essen, city)
        date = datetime.datetime(year=2017, month=2, day=10, hour=12, minute=00)

        with requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.41,6.78',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5141_678.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2017/2/10/',
                headers={'Content-Type': 'application/json'},
                content=b'[]'
            )
            with self.assertRaises(NoWeatherData):
                temperature, weather_state = meta_weather_com.coordinates2weather(
                    lat, lon, date=date
                )



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import logging

from django.db import models


log = logging.getLogger(__name__)


class GpxModelQuerySet(models.QuerySet):
    def get_by_identifier(self, identifier):
        """
        :param identifier: 'Identifier' namedtuple created here: for_runners.gpx.get_identifier
        """
        return self.get(
            start_time=identifier.start_time,
            finish_time=identifier.finish_time,
            start_latitude=identifier.start_lat,
            start_longitude=identifier.start_lon,
            finish_latitude=identifier.finish_lat,
            finish_longitude=identifier.finish_lon,
        )


class BaseGpxModelManager(models.Manager):
    def get_queryset(self):
        return GpxModelQuerySet(self.model, using=self._db)


GpxModelManager = BaseGpxModelManager.from_queryset(GpxModelQuerySet)



import shutil
from pathlib import Path
from unittest import TestCase

# https://github.com/jedie/django-tools
from django_tools.unittest_utils.django_command import DjangoCommandMixin


class ForRunnersCommandTestCase(DjangoCommandMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        # installed via setup.py entry points !
        cls.for_runners_bin = shutil.which("for_runners")
        cls.manage_bin = shutil.which("manage")

    def _call_for_runners(self, cmd, **kwargs):
        for_runners_path = Path(self.for_runners_bin)
        return self.call_manage_py(
            cmd=cmd,
            manage_dir=str(for_runners_path.parent),
            manage_py=for_runners_path.name,  # Python 3.5 needs str()
            **kwargs
        )

    def _call_manage(self, cmd, **kwargs):
        manage_path = Path(self.manage_bin)
        return self.call_manage_py(
            cmd=cmd,
            manage_dir=str(manage_path.parent),
            manage_py=manage_path.name,  # Python 3.5 needs str()
            **kwargs
        )



from for_runners_project.settings.base import *  # noqa


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

SECRET_KEY = 'No individual secret for tests ;)'

DEBUG = True

# Speedup tests by change the Password hasher:
PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

# _____________________________________________________________________________

# Skip download map via geotiler in for_runners.gpx_tools.gpxpy2map.generate_map
MAP_DOWNLOAD = False


# All tests should use django-override-storage!
# Set root to not existing path, so that wrong tests will fail:
STATIC_ROOT = '/not/exists/static/'
MEDIA_ROOT = '/not/exists/media/'



"""
    created 12.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import requests_mock
from django.test import TestCase
from django_tools.unittest_utils.assertments import assert_pformat_equal
from django_tools.unittest_utils.user import TestUserMixin
from override_storage import locmem_stats_override_storage

from for_runners.services.gpx_create import add_from_file, add_from_files, add_gpx
from for_runners.tests.fixture_files import FIXTURES_PATH, fixture_content
from for_runners.tests.utils import ClearCacheMixin


class GpxTests(TestUserMixin, ClearCacheMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.user = self._get_user(usertype="normal")

    def assert_garmin_connect_1_gpx(self, instance):
        print(instance)

        self.assertEqual(repr(instance), "<GpxModel: 2018-02-21>")

        self.assertEqual(instance.points_no, 3)
        self.assertEqual(round(instance.length, 3), 4.727)
        self.assertEqual(instance.duration_s, 2)
        self.assertEqual(round(instance.pace, 3), 7.052)
        self.assertEqual(instance.heart_rate_avg, 125)

        self.assertEqual(instance.get_short_slug(), "2018-02-21")

    def test_add_gpx(self):
        with locmem_stats_override_storage() as storage_stats, requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.44,6.62',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5144_662.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/2/21/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_2_21.json')
            )
            gpx_content = fixture_content('garmin_connect_1.gpx', mode='r')
            instance = add_gpx(gpx_content=gpx_content, user=self.user)
            self.assert_garmin_connect_1_gpx(instance)
        assert storage_stats.fields_saved == [
            ('for_runners', 'gpxmodel', 'track_svg'),
            ('for_runners', 'gpxmodel', 'gpx_file')
        ]
        assert storage_stats.fields_read == []

    def test_add_from_file(self):
        with locmem_stats_override_storage() as storage_stats, requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.44,6.62',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5144_662.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/2/21/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_2_21.json')
            )
            instance = add_from_file(
                gpx_file_file_path=FIXTURES_PATH / 'garmin_connect_1.gpx',
                user=self.user
            )
            self.assert_garmin_connect_1_gpx(instance)
        assert storage_stats.fields_saved == [
            ('for_runners', 'gpxmodel', 'track_svg'),
            ('for_runners', 'gpxmodel', 'gpx_file')
        ]
        assert storage_stats.fields_read == []

    def test_add_from_files(self):
        with locmem_stats_override_storage() as storage_stats, requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.44,6.62',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5144_662.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/2/21/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_2_21.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=52.52,13.38',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5252_1338.json')  # 4.5°C 'Light Cloud'
            )
            m.get(
                'https://www.metaweather.com/api/location/638242/2011/1/13/',
                headers={'Content-Type': 'application/json'},
                content=b'[]',  # No weather data for start.
            )
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=46.95,7.44',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_4695_744.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/784794/2011/1/15/',
                headers={'Content-Type': 'application/json'},
                content=b'[]',  # No weather data for start.
            )
            instances = [
                str(instance)
                for instance in add_from_files(
                    gpx_files_file_path=FIXTURES_PATH,
                    user=self.user,
                    skip_errors=True,
                )
            ]
            assert_pformat_equal(
                instances, ["2018-02-21", "2011-01-13"]
            )
        assert storage_stats.fields_saved == [
            ('for_runners', 'gpxmodel', 'track_svg'),
            ('for_runners', 'gpxmodel', 'gpx_file')
        ]
        assert storage_stats.fields_read == []



"""
    created 12.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

# import io
# import logging
#
# from autotask.tasks import delayed_task
# from for_runners.gpx_tools.garmin2gpxpy import garmin2gpxpy
#
#
# log = logging.getLogger(__name__)
#
#
# @delayed_task()
# def generate_gpx_map_task(object_id):
#     """
#     Delayed task to generate the map from GPX track
#     """
#     log.debug("Generate GPX Map for ID: %r", object_id)
#
#     from for_runners.models import GpxModel # import here, because of import-loop
#     gpx_instance = GpxModel.objects.get(pk=object_id)
#     log.info("Generate GPX Map for: %s" % gpx_instance)
#
#     content = gpx_instance.gpx
#     gpxpy_instance = garmin2gpxpy(content)
#
#     image, plt = generate_map(gpxpy_instance)
#
#     temp = io.BytesIO()
#     plt.savefig(temp, bbox_inches="tight")
#
#     filename = "%s.png" % gpx_instance.get_short_slug()
#
#     # Save gpx map file to model instance:
#     gpx_instance.map_image.save(filename, temp)
#
#     log.info("GPX data saved to %s, ok." % gpx_instance)



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging
from decimal import Decimal as D

from django.conf import settings
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
# https://github.com/jedie/django-tools
from django_tools.file_storage.file_system_storage import OverwriteFileSystemStorage
from django_tools.models import UpdateTimeBaseModel

# https://github.com/jedie/django-for-runners
from for_runners.gpx import GpxIdentifier, parse_gpx
from for_runners.gpx_tools.humanize import human_distance, human_duration, human_seconds
from for_runners.managers.gpx import GpxModelManager
from for_runners.model_utils import ModelAdminUrlMixin
from for_runners.models import DistanceModel, ParticipationModel


log = logging.getLogger(__name__)


def svg_upload_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/user_<id>/<filename>
    return instance.get_svg_upload_path(filename=filename)


def gpx_upload_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/user_<id>/<filename>
    return instance.get_gpx_upload_path(filename=filename)


class GpxModel(ModelAdminUrlMixin, UpdateTimeBaseModel):
    """
    inherit from ModelAdminUrlMixin:
        * get_admin_change_url()

    inherit from UpdateTimeBaseModel:
        * createtime
        * lastupdatetime
    """

    participation = models.OneToOneField(
        ParticipationModel, related_name="track", null=True, blank=True, on_delete=models.PROTECT
    )

    gpx = models.TextField(help_text="The raw gpx file content")
    gpx_file = models.FileField(
        verbose_name=_("GPX Track"),
        upload_to=gpx_upload_path,
        storage=OverwriteFileSystemStorage(create_backups=True),
        max_length=511,
        null=True,
        blank=True,
    )

    creator = models.CharField(help_text="Used device to create this track", max_length=511, null=True, blank=True)
    track_svg = models.FileField(
        verbose_name=_("Track SVG"),
        upload_to=svg_upload_path,
        storage=OverwriteFileSystemStorage(create_backups=False),
        null=True,
        blank=True,
    )

    start_time = models.DateTimeField(editable=False, help_text=_("Start time of the first segment in track"))
    start_latitude = models.FloatField(
        editable=False, help_text=_("Latitude of the first recorded point from the *.gpx file")
    )
    start_longitude = models.FloatField(
        editable=False, help_text=_("Longitude of the first recorded point from the *.gpx file")
    )
    start_temperature = models.FloatField(editable=True, null=True, blank=True, help_text=_("Temperature at start."))
    start_weather_state = models.CharField(max_length=127, null=True, blank=True, help_text="Weather state at start.")
    short_start_address = models.CharField(
        max_length=255, null=True, blank=True, help_text="The short address of the start point"
    )
    full_start_address = models.CharField(
        max_length=255, null=True, blank=True, help_text="The full address of the start point"
    )

    finish_time = models.DateTimeField(editable=False, help_text=_("End time of the last segment in track"))
    finish_latitude = models.FloatField(editable=False, help_text=_("Latitude of the finish point"))
    finish_longitude = models.FloatField(editable=False, help_text=_("Longitude of the finish point"))
    finish_temperature = models.FloatField(editable=True, null=True, blank=True, help_text=_("Temperature at finish."))
    finish_weather_state = models.CharField(
        max_length=127, null=True, blank=True, help_text="Weather state at finish."
    )
    short_finish_address = models.CharField(
        max_length=255, null=True, blank=True, help_text="The short address of the finish point"
    )
    full_finish_address = models.CharField(
        max_length=255, null=True, blank=True, help_text="The full address of the finish point"
    )

    tracked_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        editable=False,
        related_name="%(class)s_createby",
        help_text="The user that tracked this gpx entry",
        on_delete=models.PROTECT,
    )
    lastupdateby = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        editable=False,
        related_name="%(class)s_lastupdateby",
        null=True,
        blank=True,
        help_text="User as last edit this entry",
        on_delete=models.SET_NULL,
    )

    points_no = models.PositiveIntegerField(help_text=_("Number of points in GPX"), null=True, blank=True)

    length = models.PositiveIntegerField(
        help_text=_(
            "Length in meters (calculated from GPX track 3-dimensional used latitude, longitude, and elevation)"
        ),
        null=True,
        blank=True,
    )
    ideal_distance = models.ForeignKey(
        to=DistanceModel,
        on_delete=models.SET_NULL,
        related_name="tracks",
        help_text=_("Length in meters (calculated 3-dimensional used latitude, longitude, and elevation)"),
        null=True,
        blank=True,
    )

    duration_s = models.PositiveIntegerField(
        help_text=_("Duration in seconds (from the GPX track)"), editable=False, null=True, blank=True
    )
    pace = models.DecimalField(
        help_text=_("Min/km (number of minutes it takes to cover a kilometer)"),
        max_digits=4,
        decimal_places=2,  # store numbers up to 99 with a resolution of 2 decimal places
        null=True,
        blank=True,
    )

    uphill = models.IntegerField(help_text=_("Uphill elevation climbs in meters"), null=True, blank=True)
    downhill = models.IntegerField(help_text=_("Downhill elevation descent in meters"), null=True, blank=True)

    min_elevation = models.IntegerField(help_text=_("Minimum elevation in meters"), null=True, blank=True)
    max_elevation = models.IntegerField(help_text=_("Maximum elevation in meters"), null=True, blank=True)

    heart_rate_min = models.PositiveIntegerField(
        help_text=_("Minimum heart rate."), null=True, blank=True, editable=False
    )
    heart_rate_avg = models.PositiveIntegerField(
        help_text=_("Average heart rate."), null=True, blank=True, editable=False
    )
    heart_rate_max = models.PositiveIntegerField(
        help_text=_("Maximum heart rate."), null=True, blank=True, editable=False
    )

    objects = GpxModelManager()

    def save(self, *args, **kwargs):

        if self.pace is not None:
            # avoid error like:
            # Ensure that there are no more than 2 decimal places.
            self.pace = D(self.pace).quantize(D("10.00"))

        self.full_clean()
        super().save(*args, **kwargs)

        # TODO: schedule request weather info, if not set

    def _get_track_upload_path(self, *, file_extension):
        """
        e.g:
            ~/DjangoForRunnersEnv/media/gpx_track_<date>/<prefix_id>.<file_extension>
        """
        date_prefix = self.start_time.strftime("%Y_%m")
        upload_path = "/".join(
            (
                # settings.MEDIA_ROOT,
                # TODO: Use https://github.com/jedie/django-tools/tree/master/django_tools/serve_media_app
                self.tracked_by.username,
                f"gpx_track_{date_prefix}",
                f"{self.get_prefix_id()}.{file_extension}",
            )
        )
        return upload_path

    def get_svg_upload_path(self, *, filename):
        """
        SVG file will be uploaded to e.g.:
        /home/<username>/DjangoForRunnersEnv/media/gpx_track_<date>/<prefix_id>.svg
        """
        svg_upload_path = self._get_track_upload_path(file_extension="svg")
        log.debug("Ignore source filename: %r upload to: %s", filename, svg_upload_path)
        return svg_upload_path

    def get_gpx_upload_path(self, *, filename):
        """
        GPX file will be uploaded to e.g.:
        /home/<username>/DjangoForRunnersEnv/media/gpx_track_<date>/<prefix_id>.gpx
        """
        gpx_upload_path = self._get_track_upload_path(file_extension="gpx")
        log.debug("Ignore source filename: %r upload to: %s", filename, gpx_upload_path)
        return gpx_upload_path

    def svg_tag(self):
        if self.track_svg:
            html = f'<img src="{self.track_svg.url}" alt="gpx track" height="70px" width="70px" />'
            return mark_safe(html)
        return ""

    svg_tag.short_description = _("SVG")

    def svg_tag_big(self):
        if self.track_svg:
            html = f'<img src="{self.track_svg.url}" alt="gpx track" height="200px" width="200px" />'
            return mark_safe(html)
        return ""

    svg_tag_big.short_description = _("SVG")

    def start_end_address(self):
        if self.short_start_address == self.short_finish_address:
            return f"⟳ {self.short_start_address}"
        html = f"{self.short_start_address}<br>▾<br>{self.short_finish_address}"
        return mark_safe(html)

    start_end_address.short_description = _("Start/End Address")

    def get_ideal_ratio(self):
        if self.ideal_distance:
            ratio = (float(self.ideal_distance.distance_km) * 1000) / self.length
            return ratio

    def get_ideal_duration_s(self):
        ratio = self.get_ideal_ratio()
        if ratio:
            return self.duration_s * ratio

    def get_ideal_pace(self):
        ratio = self.get_ideal_ratio()
        if ratio:
            return self.pace * ratio

    def get_ideal_distance_diff_m(self):
        if self.ideal_distance:
            distance_diff_m = (float(self.ideal_distance.distance_km) * 1000) - self.length
            return distance_diff_m

    def human_ideal_length(self):
        """
        used as labels in chart.js
        """
        if self.ideal_distance:
            return self.ideal_distance.get_human_distance()
        return human_distance(self.length / 1000)

    def human_length(self):
        if self.length:
            if self.ideal_distance:
                return self.ideal_distance.get_human_distance()
            else:
                return human_distance(self.length / 1000)

    def human_length_html(self):
        """
        Enhanced version of self.human_length()
        with more information via <span title="...">
        """
        if self.length:
            length_km = self.length / 1000

            if self.ideal_distance:
                diff_km = abs(self.get_ideal_distance_diff_m() / 1000)
                html = (
                    "<span"
                    ' title="{ideal} is the standardized distance'
                    ' - GPX-Track: {gpx} (diff: {diff})"'
                    ">{ideal}</span>"
                ).format(
                    ideal=self.ideal_distance.get_human_distance(),
                    gpx=human_distance(length_km),
                    diff=human_distance(diff_km),
                )
            else:
                html = ('<span title="real distance">%s</span>') % human_distance(length_km)

            return mark_safe(html)

    human_length_html.short_description = _("Length")
    human_length_html.admin_order_field = "length"

    def get_net_duration_s(self):
        if self.participation:
            # Use net duration from event participation
            return self.participation.get_duration_s()

    def human_duration(self):
        # Use net duration from event participation
        net_duration_s = self.get_net_duration_s()
        if net_duration_s is not None:
            return human_seconds(net_duration_s)

        ideal_duration_s = self.get_ideal_duration_s()
        if ideal_duration_s:
            return human_seconds(ideal_duration_s)

        if self.duration_s:
            return human_seconds(self.duration_s)

    def human_duration_html(self):
        """
        Enhanced version of self.human_duration()
        with more information via <span title="...">
        """
        net_duration_s = self.get_net_duration_s()
        if net_duration_s is not None:
            duration_diff = self.duration_s - net_duration_s
            html = (
                "<span"
                ' title="{net} is the official net duration'
                ' - GPX-Track: {gpx} (diff: {diff})"'
                ">{net}</span>"
            ).format(
                net=human_seconds(net_duration_s),
                gpx=human_seconds(self.duration_s),
                diff=human_duration(duration_diff),
            )
            return mark_safe(html)

        ideal_duration_s = self.get_ideal_duration_s()
        if ideal_duration_s:
            duration_diff = self.duration_s - ideal_duration_s
            html = (
                "<span"
                ' title="{ideal} is the standardized duration'
                ' - GPX-Track: {gpx} (diff: {diff})"'
                ">{ideal}</span>"
            ).format(
                ideal=human_seconds(ideal_duration_s),
                gpx=human_seconds(self.duration_s),
                diff=human_duration(duration_diff),
            )
            return mark_safe(html)

        if self.duration_s:
            html = ("<span" ' title="real duration"' ">%s</span>") % human_seconds(self.duration_s)
            return mark_safe(html)

    human_duration_html.short_description = _("Duration")
    human_duration_html.admin_order_field = "duration_s"

    def human_pace(self):
        if self.pace:
            return f"{human_seconds(self.pace * 60)} min/km"

    human_pace.short_description = _("Pace")
    human_pace.admin_order_field = "pace"

    def human_weather(self):
        if not self.start_temperature:
            return "-"
        html = f"{round(self.start_temperature, 1)}°C<br/>{self.start_weather_state}"
        return mark_safe(html)

    human_weather.short_description = _("Weather")
    human_weather.admin_order_field = "start_temperature"

    def _coordinate2link(self, lat, lon):
        html = (
            "<a"
            ' href="https://nominatim.openstreetmap.org/reverse?lat={lat}&lon={lon}&format=json&addressdetails=1"'
            ' title="Reverse {lat},{lon} address with OpenStreepMap"'
            ' target="_blank"'
            ">reverse address</a>"
            "<br>"
            "<a"
            ' href="https://www.openstreetmap.org/search?query={lat}%2C{lon}"'
            ' title="OpenStreepMap at {lat},{lon}"'
            ' target="_blank"'
            ">map</a>"
        ).format(lat=lat, lon=lon)
        return mark_safe(html)

    def start_coordinate_html(self):
        """
        return HTML Links for start point.
        """
        if self.start_latitude and self.start_longitude:
            return self._coordinate2link(lat=self.start_latitude, lon=self.start_longitude)

    start_coordinate_html.short_description = _("Start coordinates")

    def finish_coordinate_html(self):
        """
        return HTML Links for finish point.
        """
        if self.finish_latitude and self.finish_longitude:
            return self._coordinate2link(lat=self.finish_latitude, lon=self.finish_longitude)

    finish_coordinate_html.short_description = _("finish coordinates")

    def point_density(self):
        """
        Calculate the "density" of the GPX signal:
        :return: float - Meters/point count
        """
        if self.length > 0:
            density = self.length / self.points_no
            return density

    def gpx_meta(self):
        gpxpy_instance = self.get_gpxpy_instance()
        attr_names = (
            "version",
            "creator",
            "name",
            "description",
            "author_name",
            "author_email",
            "author_link",
            "author_link_text",
            "author_link_type",
            "copyright_author",
            "copyright_year",
            "copyright_license",
            "link",
            "link_text",
            "link_type",
            "time",
            "keywords",
        )
        result = []
        for attr_name in attr_names:
            value = getattr(gpxpy_instance, attr_name, None)
            if value:
                result.append((attr_name, value))

        return result

    _GPXPY_CACHE = {}

    def get_gpxpy_instance(self):
        try:
            return self._GPXPY_CACHE[self.pk]
        except KeyError:
            if self.gpx:
                gpxpy_instance = parse_gpx(content=self.gpx)
                if self.pk is not None:
                    self._GPXPY_CACHE[self.pk] = gpxpy_instance
                return gpxpy_instance

    def calc_pace(self):
        duration_s = self.get_net_duration_s()
        if not duration_s:
            duration_s = self.get_ideal_duration_s()
        if not duration_s:
            duration_s = self.duration_s
        if not duration_s:
            return None

        if self.ideal_distance:
            distance_km = int(self.ideal_distance.distance_km)
        else:
            distance_km = self.length / 1000

        try:
            pace = (duration_s / 60) / distance_km
        except ZeroDivisionError:
            # FIXME
            log.exception(f"Error calculate pace with duration {duration_s!r}sec and distance {distance_km}km!")
        else:
            if pace > 99 or pace < 0:
                log.error("Pace out of range: %f", pace)
            else:
                self.pace = pace

    def short_name(self, start_time=True):
        if self.pk is None:
            return "new, unsaved GPX Track"

        if start_time:
            parts = [self.start_time.strftime("%Y-%m-%d")]
        else:
            parts = []

        if self.participation:
            parts.append(self.participation.event.name)
        else:
            parts.append(self.short_start_address)
        result = " ".join(str(part) for part in parts if part)
        if result:
            return result
        return f"GPX Track ID:{self.pk}"

    def get_short_slug(self):
        name = self.short_name()
        return slugify(name)

    def get_prefix_id(self):
        gpxpy_instance = self.get_gpxpy_instance()
        prefix_id = GpxIdentifier(gpxpy_instance).get_prefix_id()
        return prefix_id

    def __str__(self):
        # return self.get_prefix_id()
        return self.short_name()

    class Meta:
        verbose_name = _("GPX Track")
        verbose_name_plural = _("GPX Tracks")
        unique_together = (
            ("start_time", "start_latitude", "start_longitude", "finish_time", "finish_latitude", "finish_longitude"),
        )
        ordering = ("-start_time", "-pk")



"""
    created 02.04.2019 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2019 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging

from django.core.files.base import ContentFile


log = logging.getLogger(__name__)


def save_gpx_file(*, gpx_track, force=False):
    """
    Save the gpx track to disk by "attach" it to GpxModel.gpx_file

    :param gpx_track: for_runners.models.gpx.GpxModel instance
    :param gpxpy_instance: gpxpy.parser.GPXParser instance
    :param force: don't check if gpx track already exists
    :return: gpx
    """
    log.debug("Save gpx track to disk")

    if gpx_track.gpx_file:
        # gpx file already exists.
        if not force:
            log.info("Don't recreate existing gpx file")
            return

    content = ContentFile(gpx_track.gpx)

    # https://docs.djangoproject.com/en/2.0/ref/models/fields/#django.db.models.fields.files.FieldFile.save
    gpx_track.gpx_file.save(
        name="temp.gpx", content=content, save=False  # real file path will be set in self.get_gpx_upload_path()
    )
    log.debug(f"gpx file created: {gpx_track.gpx_file!r}")



"""
    https://docs.djangoproject.com/en/2.0/ref/applications/#configuring-applications-ref

    created 04.07.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.apps import AppConfig


class ForRunnersConfig(AppConfig):
    name = "for_runners"
    verbose_name = "ForRunners"



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import collections
import logging
from decimal import Decimal as D

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
from django.template.response import TemplateResponse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

# https://github.com/jedie/django-for-runners
from for_runners.admin.utils import BaseChangelistView
from for_runners.gpx_tools.humanize import convert_cash_values, human_distance, human_duration, human_seconds
from for_runners.models import EventLinkModel, EventModel
from for_runners.models.event import CostModel, ParticipationModel


log = logging.getLogger(__name__)


@admin.register(EventLinkModel)
class EventLinkModelAdmin(admin.ModelAdmin):
    list_display = ("event", "link_html")
    list_filter = ("event",)
    date_hierarchy = "event__start_date"
    search_fields = ("url", "text", "title", "event__name")


class LinkModelInline(admin.TabularInline):
    model = EventLinkModel
    extra = 2
    min_num = 0
    max_num = None
    fields = ("url", "text", "title")


class HasTracksFilter(admin.SimpleListFilter):
    title = _("has GPX tracks")
    parameter_name = "tracks"

    def lookups(self, request, model_admin):
        return (("y", _("yes")), ("n", _("no")))

    def queryset(self, request, queryset):
        if self.value() == "y":
            return queryset.exclude(participations__tracks__isnull=True)  # FIXME!
        if self.value() == "n":
            return queryset.filter(participations__tracks__isnull=True)  # FIXME!


class EventStatistics:
    KEY_DISTANCE = "_distance"
    KEY_DURATION = "_duration"

    def __init__(self):
        self.convert_map = {self.KEY_DISTANCE: human_distance, self.KEY_DURATION: human_duration}

    def get(self, events):
        raw_person_data = collections.defaultdict(collections.Counter)

        for event in events:
            # print(event)
            for participation in event.participations.all():
                # print(participation.person)
                participation_data = {
                    "_count": 1,
                    self.KEY_DISTANCE: participation.distance_km or 0,
                    self.KEY_DURATION: participation.get_duration_s() or 0,
                }

                for cost in participation.costs.all():
                    # print(cost)
                    participation_data[cost.name] = cost.amount

                raw_person_data[participation.person.username] += collections.Counter(participation_data)

        # turn defaultdict and counter to normal dict and convert some values:

        total_costs = collections.Counter()

        # pprint(raw_person_data)
        person_data = {}
        for username, counter_data in raw_person_data.items():
            user_data = []
            counter_data = dict(counter_data)
            for key, value in sorted(counter_data.items()):
                try:
                    func = self.convert_map[key]
                except KeyError:
                    pass
                else:
                    value = func(value)

                if key.startswith("_"):
                    # internal key
                    key = key[1:]
                else:
                    # costs item
                    # collect total costs:
                    total_costs += collections.Counter({"total": value, key: value})
                    # convert decimal field:
                    value = convert_cash_values(value)

                user_data.append((key, value))
            person_data[username] = user_data

        total_costs = dict(total_costs)
        total = total_costs.pop("total")  # add total as lates item
        total_costs = [(name, convert_cash_values(amount)) for name, amount in sorted(total_costs.items())]
        total_costs.append((_("total"), convert_cash_values(total)))

        # pprint(person_data)
        # pprint(total_costs)
        return person_data, total_costs


class StatisticsView(BaseChangelistView):

    template_name = "admin/for_runners/eventmodel/statistics.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        events = self.change_list.queryset  # filtered EventModel queryset form GpxModelChangeList
        # print("Events:", events.count())

        person_data, total_costs = EventStatistics().get(events)

        context.update(
            {
                "title": _("Event Statistics"),
                "person_data": person_data,
                "total_costs": total_costs,
                "user": self.request.user,
                "opts": EventModel._meta,
            }
        )
        return context


class EventModelChangeList(ChangeList):
    def get_results(self, request):
        super().get_results(request)

        view = StatisticsView.as_view()
        response = view(request, self)
        assert isinstance(response, TemplateResponse), f"Method {view} didn't return a TemplateResponse!"
        self.statistics = response.rendered_content


@admin.register(EventModel)
class EventModelAdmin(admin.ModelAdmin):
    def gpx_tracks_change_form_links(self, obj):
        """
        Link to GPX Tracks in object change view
        """
        html = []
        participations = obj.participations.all()
        for participation in participations:
            gpx_track = participation.track
            if gpx_track:
                change_url = gpx_track.get_admin_change_url()
                html.append(
                    '{username}: <a href="{url}">{length} <strong>{duration}</strong> {pace}</a>'.format(
                        username=participation.person.username,
                        url=change_url,
                        length=gpx_track.human_length_html(),
                        duration=gpx_track.human_duration_html(),
                        pace=gpx_track.human_pace(),
                    )
                )

        html = "<br>".join(html)
        html = mark_safe(html)
        return html

    gpx_tracks_change_form_links.short_description = _("GPX Tracks")

    def gpx_tracks_change_list_links(self, obj):
        """
        Link to GPX Tracks in admin change list
        """
        html = []
        participations = obj.participations.all()
        for participation in participations:
            gpx_track = participation.track
            if gpx_track:
                change_url = gpx_track.get_admin_change_url()
                html.append(
                    '<a href="{url}">{length} <strong>{duration}</strong> {pace}</a>'.format(
                        url=change_url,
                        length=gpx_track.human_length_html(),
                        duration=gpx_track.human_duration_html(),
                        pace=gpx_track.human_pace(),
                    )
                )

        html = "<br>".join(html)
        html = mark_safe(html)
        return html

    gpx_tracks_change_list_links.short_description = _("GPX Tracks")

    def participations(self, obj):
        html = []
        participations = obj.participations.all()
        for participation in participations:
            change_url = participation.get_admin_change_url()
            html.append(
                f'<a href="{change_url}">{participation.person.username} ({participation.get_human_distance()})</a>'
            )

        html = "<br>".join(html)
        html = mark_safe(html)
        return html

    participations.short_description = _("Participations")

    readonly_fields = ("gpx_tracks_change_form_links",)
    fieldsets = (
        (_("Event Name"), {"fields": (("no", "name", "start_date"), "discipline", "gpx_tracks_change_form_links")}),
    )

    list_display = (
        "verbose_name",
        "participations",
        "gpx_tracks_change_list_links",
        "links_html",
        "start_date",
        "discipline",
    )
    date_hierarchy = "start_date"
    list_filter = (HasTracksFilter, "participations__person")
    list_display_links = ("verbose_name",)
    search_fields = ("name",)
    inlines = [LinkModelInline]

    def get_changelist(self, request, **kwargs):
        """
        Returns the ChangeList class for use on the changelist page.
        """
        return EventModelChangeList


class CostModelInline(admin.TabularInline):
    model = CostModel
    extra = 2
    min_num = 0
    max_num = None


@admin.register(ParticipationModel)
class ParticipationModelAdmin(admin.ModelAdmin):
    def start_date(self, obj):
        return obj.event.start_date

    start_date.admin_order_field = "event__start_date"

    def event_name(self, obj):
        return obj.event.short_name()

    event_name.short_description = _("Event")
    event_name.admin_order_field = "event"

    def human_duration(self, obj):
        if obj.duration:
            return human_seconds(obj.get_duration_s())

    human_duration.short_description = _("Duration")
    human_duration.admin_order_field = "duration"

    def pace(self, obj):
        pace_s = obj.get_pace_s()
        if pace_s:
            return f"{human_seconds(pace_s)} min/km"

    pace.short_description = _("Pace")

    def costs(self, obj):
        parts = []
        total = D("0.00")
        for entry in obj.costs.all():
            total += entry.amount
            parts.append(str(entry))

        if not parts:
            return ""
        elif len(parts) == 1:
            return f"{total:.2f}€"
        else:
            return f"{total:.2f}€ ({' '.join(parts)})"

    list_display = (
        "start_date",
        "event_name",
        "distance_km",
        "finisher_count",
        "person",
        "costs",
        "start_number",
        "human_duration",
        "pace",
    )
    list_display_links = ("event_name",)
    list_filter = ("person", "distance_km")
    date_hierarchy = "event__start_date"
    search_fields = ("person__username", "event__name", "event__start_date")
    inlines = [CostModelInline]



"""
    TODO: Move to django-tools
"""

from django.urls import reverse


class ModelAdminUrlMixin:
    def get_admin_change_url(self):
        return reverse(f"admin:{self._meta.app_label}_{self._meta.model_name}_change", args=(self.pk,))



#
# Hardcore Monkey patch to find not mocked requests ;)
# But this will also break selenium tests, too.
#
# import socket
#
# class SocketMock:
#     def __call__(self, *args, **kwargs):
#         raise NotImplementedError(
#             'Socket is monkeypatched in Tests!'
#         )
#
# socket.socket = SocketMock()



from pathlib import Path

import requests_mock
from bx_django_utils.test_utils.html_assertion import HtmlAssertionMixin
from django.contrib.auth.models import User
from django.test import TestCase
from model_bakery import baker
from override_storage import locmem_stats_override_storage

from for_runners import __version__
from for_runners.models import GpxModel
from for_runners.tests.fixture_files import FIXTURES_PATH, fixture_content


class ForRunnerAdminTests(HtmlAssertionMixin, TestCase):
    """
    Special for runners admin tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.superuser = baker.make(User, username='test-superuser', is_staff=True, is_active=True, is_superuser=True)

    def test_staff_upload(self):
        self.client.force_login(self.superuser)

        gpx_file_path1 = Path(FIXTURES_PATH, "garmin_connect_1.gpx")
        gpx_file_path2 = Path(FIXTURES_PATH, "no_track_points.gpx")

        with gpx_file_path1.open("rb") as file1, gpx_file_path2.open("rb") as file2, \
                locmem_stats_override_storage() as storage_stats, requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.44,6.62',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5144_662.json'),
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/2/21/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_2_21.json'),
            )
            m.get(
                (
                    'https://nominatim.openstreetmap.org/reverse?lat=51.43789&lon=6.61701'
                    '&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('osm_5143789_661701.json'),
            )
            m.get(  # Start point lat=51.437889290973544 - lat=6.617012657225132
                (
                    'https://nominatim.openstreetmap.org/reverse?lat=51.43785&lon=6.61701'
                    '&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('osm_5143789_661701.json'),
            )
            m.get(  # End point lat=51.437847297638655 - lat=6.6170057002455
                (
                    'https://nominatim.openstreetmap.org/reverse?lat=51.43789&lon=6.61701'
                    '&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('osm_5143785_661701.json'),
            )
            response = self.client.post(
                "/en/admin/for_runners/gpxmodel/upload/",
                data={"gpx_files": [file1, file2]},
                HTTP_ACCEPT_LANGUAGE="en",
            )
            # debug_response(response)
            assert storage_stats.fields_saved == [
                ('for_runners', 'gpxmodel', 'track_svg'),
                ('for_runners', 'gpxmodel', 'gpx_file'),
            ]
            assert storage_stats.fields_read == []

            tracks = GpxModel.objects.all()
            self.assertEqual(tracks.count(), 1)
            new_track = tracks[0]

            self.assertRedirects(
                response,
                expected_url=f"/en/admin/for_runners/gpxmodel/{new_track.pk:d}/change/",
                status_code=302,
                target_status_code=200,
                fetch_redirect_response=False,
            )

            response = self.client.get("/en/admin/for_runners/gpxmodel/", HTTP_ACCEPT_LANGUAGE="en")
            # print(repr(self.get_messages(response)))
            self.assert_html_parts(
                response,
                parts=(
                    f"<title>Select GPX Track to change | Django-ForRunners v{__version__}</title>",
                    # "Process garmin_connect_1.gpx...",
                    # "Created: 2018-02-21 14:30:50 Moers",
                    #
                    # "Process no_track_points.gpx...",
                    # "Error process GPX data: Can't get first track",
                    '<td class="field-human_pace">7:03 min/km</td>',
                    '<p class="paginator">1 GPX Track</p>',
                ),
            )
            self.assertTemplateUsed(response, template_name="admin/for_runners/gpxmodel/change_list.html")
            self.assert_messages(
                response,
                expected_messages=[
                    "Process garmin_connect_1.gpx...",
                    "Created: 2018-02-21 Moers",
                    "Process no_track_points.gpx...",
                    "Error process GPX data: Can't get first track",
                ],
            )

            assert storage_stats.fields_read == []

    def test_add_view_redirect_to_upload(self):
        self.client.force_login(self.superuser)
        response = self.client.get("/en/admin/for_runners/gpxmodel/add/", HTTP_ACCEPT_LANGUAGE="en")
        self.assertRedirects(
            response,
            expected_url="/en/admin/for_runners/gpxmodel/upload/",
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=True,
        )



import os
import shutil
import subprocess
from pathlib import Path

from creole.setup_utils import update_rst_readme
from django.conf import settings
from django.core.cache import cache
from django.test import TestCase

import for_runners


PACKAGE_ROOT = Path(for_runners.__file__).parent.parent.parent


def assert_file_contains_string(file_path, string):
    with file_path.open('r') as f:
        for line in f:
            if string in line:
                return
    raise AssertionError(f'File {file_path} does not contain {string!r} !')


def test_version(package_root=None, version=None):
    if package_root is None:
        package_root = PACKAGE_ROOT

    if version is None:
        version = for_runners.__version__

    if 'dev' not in version and 'rc' not in version:
        version_string = f'v{version}'

        assert_file_contains_string(
            file_path=Path(package_root, 'README.creole'),
            string=version_string
        )

        assert_file_contains_string(
            file_path=Path(package_root, 'README.rst'),
            string=version_string
        )

    assert_file_contains_string(
        file_path=Path(package_root, 'pyproject.toml'),
        string=f'version = "{version}"'
    )

    assert_file_contains_string(
        file_path=Path(package_root, 'deployment', 'project.env'),
        string=f'PROJECT_VERSION={version}'
    )


def test_poetry_check(package_root=None):
    if package_root is None:
        package_root = PACKAGE_ROOT

    poerty_bin = shutil.which('poetry')

    output = subprocess.check_output(
        [poerty_bin, 'check'],
        universal_newlines=True,
        env=os.environ,
        stderr=subprocess.STDOUT,
        cwd=str(package_root),
    )
    print(output)
    assert output == 'All set!\n'


def test_update_rst_readme(capsys):
    rest_readme_path = update_rst_readme(
        package_root=PACKAGE_ROOT, filename='README.creole'
    )
    captured = capsys.readouterr()
    assert captured.out == 'Generate README.rst from README.creole...nothing changed, ok.\n'
    assert captured.err == ''
    assert isinstance(rest_readme_path, Path)
    assert str(rest_readme_path).endswith('/README.rst')


class ProjectSettingsTestCase(TestCase):
    def test_project_path(self):
        project_path = settings.PROJECT_PATH
        assert project_path.is_dir()
        assert Path(project_path, 'for_runners').is_dir()
        assert Path(project_path, 'for_runners_project').is_dir()

    def test_template_dirs(self):
        assert len(settings.TEMPLATES) == 1
        dirs = settings.TEMPLATES[0].get('DIRS')
        assert len(dirs) == 1
        template_path = Path(dirs[0]).resolve()
        assert template_path.is_dir()

    def test_cache(self):
        # django cache should work in tests, because some tests "depends" on it
        cache_key = 'a-cache-key'
        assert cache.get(cache_key) is None
        cache.set(cache_key, 'the cache content', timeout=1)
        assert cache.get(cache_key) == 'the cache content'
        cache.delete(cache_key)
        assert cache.get(cache_key) is None



# Generated by Django 2.0.7 on 2018-07-10 19:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL), ("for_runners", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="CostModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "createtime",
                    models.DateTimeField(default=django.utils.timezone.now, editable=False, help_text="Create time"),
                ),
                (
                    "lastupdatetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False, help_text="Time of the last change."
                    ),
                ),
                ("name", models.CharField(help_text="Your start number", max_length=15)),
                (
                    "amount",
                    models.DecimalField(decimal_places=2, help_text="How much did you pay for this?", max_digits=8),
                ),
            ],
            options={"verbose_name": "Participation Cost", "verbose_name_plural": "Participation Costs"},
        ),
        migrations.CreateModel(
            name="ParticipationModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "createtime",
                    models.DateTimeField(default=django.utils.timezone.now, editable=False, help_text="Create time"),
                ),
                (
                    "lastupdatetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False, help_text="Time of the last change."
                    ),
                ),
                (
                    "duration",
                    models.TimeField(
                        blank=True,
                        help_text="You officially measured finisher time",
                        null=True,
                        verbose_name="Duration",
                    ),
                ),
                (
                    "start_number",
                    models.CharField(blank=True, help_text="Your start number", max_length=15, null=True),
                ),
                (
                    "finisher_count",
                    models.PositiveIntegerField(
                        blank=True, help_text="Number of participants who have finished in your discipline", null=True
                    ),
                ),
                (
                    "distance",
                    models.ForeignKey(
                        help_text="Length in meters of your event participation",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="event_participations",
                        to="for_runners.DistanceModel",
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="participations",
                        to="for_runners.EventModel",
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="The person who participated to this competition.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"verbose_name": "Event Participation", "verbose_name_plural": "Event Participations"},
        ),
        migrations.AddField(
            model_name="costmodel",
            name="participation",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, related_name="costs", to="for_runners.ParticipationModel"
            ),
        ),
    ]



"""
    https://django-import-export.readthedocs.io/en/latest/getting_started.html
"""

from import_export import resources
from import_export.fields import Field

# https://github.com/jedie/django-for-runners
from for_runners.models import GpxModel


class GpxModelResource(resources.ModelResource):
    """
    TODO: Use it in for_runners.management.commands.backup.Command
    """

    start_time = Field()
    name = Field()
    event = Field()
    duration = Field()
    pace = Field()
    heart_rate = Field()
    temperature = Field()
    weather = Field()
    username = Field()

    def dehydrate_start_time(self, track):
        return track.start_time

    def dehydrate_name(self, track):
        return track.short_name(start_time=False)

    def dehydrate_event(self, track):
        # Note: track.participation.event.verbose_name() is used in short_name() ;)
        return "x" if track.participation else ""

    def dehydrate_length(self, track):
        return round(track.length / 1000, 2)

    def dehydrate_duration(self, track):
        return track.human_duration()

    def dehydrate_pace(self, track):
        return track.human_pace()

    def dehydrate_heart_rate(self, track):
        if track.heart_rate_avg:
            return "%i b/m" % track.heart_rate_avg

    def dehydrate_temperature(self, track):
        if track.start_temperature:
            return "%i°C" % round(track.start_temperature, 1)

    def dehydrate_weather(self, track):
        if track.start_temperature:
            return track.start_weather_state

    def dehydrate_username(self, track):
        return track.tracked_by.username

    class Meta:
        model = GpxModel
        fields = (
            "start_time",
            "name",
            "event",
            "length",
            "duration",
            "pace",
            "heart_rate",
            "temperature",
            "weather",
            "creator",
            "username",
        )



import os
import unittest

from bx_django_utils.test_utils.html_assertion import HtmlAssertionMixin
from django.contrib.auth.models import User
from django.test import TestCase
from django_tools.selenium.chromedriver import chromium_available
from django_tools.selenium.django import (
    SeleniumChromiumStaticLiveServerTestCase,
    SeleniumFirefoxStaticLiveServerTestCase,
)
from django_tools.selenium.geckodriver import firefox_available
from model_bakery import baker

from for_runners import __version__


class AdminAnonymousTests(HtmlAssertionMixin, TestCase):
    """
    Anonymous will be redirected to the login page.
    """

    def test_login_en(self):
        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assertRedirects(response, expected_url="/en/admin/login/?next=/en/admin/")

    def test_login_de(self):
        response = self.client.get("/de/admin/", HTTP_ACCEPT_LANGUAGE="de")
        self.assertRedirects(response, expected_url="/de/admin/login/?next=/de/admin/")


class AdminLoggedinTests(HtmlAssertionMixin, TestCase):
    """
    Some basics test with the django admin
    """

    @classmethod
    def setUpTestData(cls):
        cls.superuser = baker.make(User, username='superuser', is_staff=True, is_active=True, is_superuser=True)
        cls.staffuser = baker.make(User, username='staff_test_user', is_staff=True, is_active=True, is_superuser=False)

    def test_staff_admin_index(self):
        self.client.force_login(self.staffuser)

        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assert_html_parts(
            response,
            parts=(
                f"<title>Site administration | Django-ForRunners v{__version__}</title>",
                "<h1>Site administration</h1>",
                "<strong>staff_test_user</strong>",
                "<p>You don't have permission to view or edit anything.</p>",
            ),
        )
        self.assertTemplateUsed(response, template_name="admin/index.html")

    def test_superuser_admin_index(self):
        self.client.force_login(self.superuser)
        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assert_html_parts(
            response,
            parts=(
                "Django-ForRunners",
                "<strong>superuser</strong>",
                "Site administration",
                "/admin/auth/group/add/",
                "/admin/auth/user/add/",
            ),
        )
        self.assertTemplateUsed(response, template_name="admin/index.html")


@unittest.skipIf('CI' in os.environ, 'Skip, selenium tests does not work on CI run!')
@unittest.skipUnless(chromium_available(), "Skip because Chromium is not available!")
class AdminChromiumTests(SeleniumChromiumStaticLiveServerTestCase):
    def test_admin_login_page(self):
        self.driver.get(self.live_server_url + "/admin/login/")
        self.assert_equal_page_title(f"Log in | Django-ForRunners v{__version__}")
        self.assert_in_page_source('<form action="/en/admin/login/" method="post" id="login-form">')
        self.assert_no_javascript_alert()


@unittest.skipIf('CI' in os.environ, 'Skip, selenium tests does not work on CI run!')
@unittest.skipUnless(firefox_available(), "Skip because Firefox is not available!")
class AdminFirefoxTests(SeleniumFirefoxStaticLiveServerTestCase):
    def test_admin_login_page(self):
        self.driver.get(self.live_server_url + "/admin/login/")
        self.assert_equal_page_title(f"Log in | Django-ForRunners v{__version__}")
        self.assert_in_page_source('<form action="/en/admin/login/" method="post" id="login-form">')
        self.assert_no_javascript_alert()



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging
from urllib.parse import urlparse

from django.conf import settings
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django_tools.models import UpdateInfoBaseModel, UpdateTimeBaseModel

from for_runners.gpx_tools.humanize import human_distance, human_seconds
from for_runners.model_utils import ModelAdminUrlMixin
from for_runners.models import DisciplineModel


log = logging.getLogger(__name__)


def human_url(url):
    scheme, netloc, url, params, query, fragment = urlparse(url)
    text = netloc + url
    text = text.strip("/")
    if text.startswith("www."):
        text = text[4:]
    return text


class LinkModelBase(UpdateTimeBaseModel):
    """
    inherit and automatically set from UpdateTimeBaseModel:
     * createtime
     * lastupdatetime
    """

    url = models.URLField(help_text=_("Link URL"))
    text = models.CharField(
        max_length=127, help_text=_("Link text (leave empty to generate it from url)"), null=True, blank=True
    )
    title = models.CharField(
        max_length=127, help_text=_("Link title (leave empty to generate it from url)"), null=True, blank=True
    )

    def get_text(self):
        return self.text or human_url(self.url)

    def get_title(self):
        return self.title or self.url

    def link_html(self):
        html = ('<a href="{url}" title="{title}" target="_blank">' "{text}" "</a>").format(
            url=self.url, title=self.get_title(), text=self.get_text()
        )
        return mark_safe(html)

    link_html.short_description = _("Link")

    def save(self, *args, **kwargs):
        if self.text is None:
            self.text = human_url(self.url)

        self.full_clean()
        super().save(*args, **kwargs)

    def __str__(self):
        return self.get_text()

    class Meta:
        abstract = True


class EventModel(ModelAdminUrlMixin, UpdateInfoBaseModel):
    """
    inherit from ModelAdminUrlMixin:
        * get_admin_change_url()

    inherit from UpdateInfoBaseModel:
        * createtime
        * lastupdatetime
        * createby
        * lastupdateby
    """

    no = models.PositiveIntegerField(help_text=_("Sequential number of the event"), null=True, blank=True)
    name = models.CharField(max_length=255, help_text=_("Name of the event"))
    start_date = models.DateField(help_text=_("Start date of the run"), null=True, blank=True)
    discipline = models.ForeignKey(DisciplineModel, on_delete=models.SET_NULL, null=True)

    def short_name(self):
        parts = []
        if self.no:
            parts.append("%i." % self.no)
        parts.append(self.name)
        result = " ".join(part for part in parts if part)
        return result

    def verbose_name(self):
        parts = [self.short_name()]
        if self.start_date:
            year = self.start_date.strftime("%Y")
            if year not in self.name:
                parts.append(year)
        result = " ".join(part for part in parts if part)
        return result

    verbose_name.short_description = _("Event Name")
    verbose_name.admin_order_field = "name"

    def links_html(self):
        links = []
        for link in self.links.all():
            links.append(link.link_html())
        html = "<br />".join(links)
        return mark_safe(html)

    links_html.short_description = _("Links")

    def __str__(self):
        return self.verbose_name()

    class Meta:
        verbose_name = _("Event")
        verbose_name_plural = _("Events")
        ordering = ("-start_date", "-pk")


class EventLinkModel(LinkModelBase):
    event = models.ForeignKey(EventModel, related_name="links", on_delete=models.CASCADE)


class ParticipationModel(ModelAdminUrlMixin, UpdateTimeBaseModel):
    """
    inherit from ModelAdminUrlMixin:
        * get_admin_change_url()

    inherit from UpdateTimeBaseModel:
        * createtime
        * lastupdatetime
    """

    event = models.ForeignKey(EventModel, related_name="participations", on_delete=models.CASCADE)
    person = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="+",
        help_text="The person who participated to this competition.",
        on_delete=models.CASCADE,
    )
    distance_km = models.DecimalField(
        help_text=_("Official track length in kilometer."),
        # store numbers up to 999 with a resolution of 4 decimal places
        max_digits=7,
        decimal_places=4,
    )
    duration = models.TimeField(
        verbose_name=_("Duration"), help_text=_("You officially measured finisher time"), null=True, blank=True
    )
    start_number = models.CharField(max_length=15, help_text=_("Your start number"), null=True, blank=True)
    finisher_count = models.PositiveIntegerField(
        help_text=_("Number of participants who have finished in your discipline"), null=True, blank=True
    )

    def get_human_distance(self):
        return human_distance(self.distance_km)

    get_human_distance.short_description = _("Distance")
    get_human_distance.admin_order_field = "distance_km"

    def get_duration_s(self):
        """
        :return: duration in seconds
        """
        if self.duration:
            # FIXME: Is there really no easier way to do this?
            duration = self.duration.second
            duration += self.duration.minute * 60
            duration += self.duration.hour * 60 * 60
            return duration

    def human_duration(self):
        if self.duration:
            return human_seconds(self.get_duration_s())

    def get_pace_s(self):
        if self.duration:
            pace = (self.get_duration_s() / 60) / float(self.distance_km)
            if pace > 99 or pace < 0:
                log.error("Pace out of range: %f", pace)
            else:
                return pace * 60

    def verbose_name(self):
        parts = [self.event.verbose_name(), "-", self.person.username, "-", self.get_human_distance()]
        if self.duration:
            parts.append(f"in {self.human_duration()}")

        result = " ".join(part for part in parts if part)
        return result

    verbose_name.short_description = _("Event Name")
    verbose_name.admin_order_field = "name"

    def __str__(self):
        return self.verbose_name()

    class Meta:
        verbose_name = _("Event Participation")
        verbose_name_plural = _("Event Participations")
        ordering = ("-event__start_date", "person")


class CostModel(UpdateTimeBaseModel):
    """
    inherit and automatically set from UpdateTimeBaseModel:
     * createtime
     * lastupdatetime
    """

    participation = models.ForeignKey(ParticipationModel, related_name="costs", on_delete=models.CASCADE)
    name = models.CharField(max_length=15, help_text=_("The name of the item you dis pay for"))
    amount = models.DecimalField(
        help_text=_("How much did you pay for this?"),
        max_digits=8,
        decimal_places=2,  # store numbers up to 99 with a resolution of 2 decimal places
    )

    def __str__(self):
        return f"{self.name}: {self.amount}"

    class Meta:
        verbose_name = _("Participation Cost")
        verbose_name_plural = _("Participation Costs")



"""
    created 25.08.2019 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2019 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging

from django.db import IntegrityError, transaction
# https://github.com/jedie/django-tools
from django_tools.unittest_utils.assertments import assert_is_dir, assert_is_file
from gpxpy.gpx import GPXException

# https://github.com/jedie/django-for-runners
from for_runners.exceptions import GpxDataError
from for_runners.gpx import get_identifier, parse_gpx
from for_runners.models import GpxModel


log = logging.getLogger(__name__)


def add_gpx(*, gpx_content, user):
    """
    Create a new for_runners.models.GpxModel entry

    :param gpx_content: String content of the new gpx file
    :return: GpxModel instance
    """
    try:
        gpxpy_instance = parse_gpx(gpx_content)
    except GPXException as err:
        log.exception(f"Invalid GPX Data: {err}")
        return

    identifier = get_identifier(gpxpy_instance)

    try:
        instance = GpxModel.objects.get_by_identifier(identifier)
    except GpxModel.DoesNotExist:
        log.debug("Create new track for user: %s", user)
        instance = GpxModel.objects.create(gpx=gpx_content, tracked_by=user)
        return instance
    else:
        if instance.tracked_by != user:
            log.error("Skip existing track: %s (Tracked by: %s)", instance, instance.tracked_by)
        else:
            log.info("Skip existing track: %s", instance)
        return


def add_from_file(*, gpx_file_file_path, user):
    """
    Read content from gpx file <gpx_files_file_path> and add to <user>
    """
    assert_is_file(gpx_file_file_path)

    log.info(f'Add GPX file: {gpx_file_file_path}')
    with gpx_file_file_path.open("r") as f:
        gpx_content = f.read()

    return add_gpx(gpx_content=gpx_content, user=user)


def add_from_files(*, gpx_files_file_path, user, skip_errors=True):
    """
    Add all *.gpx files from <gpx_files_file_path> to <user>
    """
    assert_is_dir(gpx_files_file_path)

    gpx_files = gpx_files_file_path.glob("**/*.gpx")
    for gpx_file_file_path in sorted(gpx_files):
        try:
            with transaction.atomic():
                instance = add_from_file(gpx_file_file_path=gpx_file_file_path, user=user)
        except (IntegrityError, GpxDataError) as err:
            log.error("Skip .gpx file: %s", err)
            if not skip_errors:
                raise
        else:
            if instance:
                yield instance



from pathlib import Path

import requests_mock
from django.core.management import call_command
from django.test import TestCase
from django_tools.unittest_utils.assertments import assert_is_dir, assert_pformat_equal
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer
from django_tools.unittest_utils.user import TestUserMixin
from override_storage import locmem_stats_override_storage

import for_runners
from for_runners.management.commands import import_gpx
from for_runners.models import GpxModel
from for_runners.tests.fixture_files import fixture_content
from for_runners.tests.utils import ClearCacheMixin


# TODO:
# class ImportCommandTestCase(ForRunnersCommandTestCase):
#     def test_in_help(self):
#         output = self._call_manage(["--help"])
#         print(output)
#         self.assertIn("Available subcommands:", output)
#         self.assertIn("[for_runners]", output)
#         self.assertIn("import_gpx", output)
#
#     def test_import_help(self):
#         output = self._call_manage(["import_gpx", "--help"])
#         print(output)
#         self.assertIn("usage: manage import_gpx", output)
#         self.assertIn("Path to *.gpx files", output)
#
#     def test_import_no_username_given(self):
#         output = self._call_manage(["import_gpx", "/foo/bar"], excepted_exit_code=2)
#         print(output)
#         self.assertIn("following arguments are required: --username", output)
#
#     def test_import_wrong_username(self):
#         output = self._call_manage(
#             ["import_gpx", "--username", "NotExistingUser", "/foo/bar"], excepted_exit_code=3
#         )
#         print(output)
#         self.assertIn("ERROR getting user 'NotExistingUser'", output)
#         self.assertIn("Existing usernames are", output)
#         self.assertIn("test", output)
#
#     def test_import_wrong_path(self):
#         output = self._call_manage(
#             ["import_gpx", "--username", "test", "/foo/bar"], excepted_exit_code=4
#         )
#         print(output)
#         self.assertIn("ERROR: Given path '/foo/bar' is not a existing directory!", output)


class ImportTestCase(TestUserMixin, ClearCacheMixin, TestCase):
    def test_import(self):
        test_username = "normal_test_user"

        assert GpxModel.objects.filter(tracked_by__username=test_username).count() == 0

        base_path = Path(for_runners.__file__).parent
        fixture_files_path = Path(base_path, "tests/fixture_files")
        assert_is_dir(fixture_files_path)

        with StdoutStderrBuffer() as buff, \
                locmem_stats_override_storage() as storage_stats, \
                requests_mock.mock() as m:
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=51.44,6.62',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5144_662.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/648820/2018/2/21/',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_location_648820_2018_2_21.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=52.52,13.38',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5252_1338.json')  # 4.5°C 'Light Cloud'
            )
            m.get(
                'https://www.metaweather.com/api/location/638242/2011/1/13/',
                headers={'Content-Type': 'application/json'},
                content=b'[]',  # No weather data for start.
            )
            m.get(
                'https://www.metaweather.com/api/location/search/?lattlong=46.95,7.44',
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_4695_744.json')
            )
            m.get(
                'https://www.metaweather.com/api/location/784794/2011/1/15/',
                headers={'Content-Type': 'application/json'},
                content=b'[]',  # No weather data for start.
            )
            m.get(
                (
                    'https://nominatim.openstreetmap.org/reverse'
                    '?lat=0.0&lon=0.0&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('nominatim_osm_reverse_0_0.json')
            )
            m.get(
                (
                    'https://nominatim.openstreetmap.org/reverse'
                    '?lat=0.0&lon=180.0&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=b'{"error":"Unable to geocode"}'
            )
            m.get(
                (
                    'https://nominatim.openstreetmap.org/reverse'
                    '?lat=51.437889290973544&lon=6.617012657225132&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('nominatim_osm_reverse_0_0.json')
            )
            call_command(import_gpx.Command(), "--username", test_username, str(fixture_files_path))
            assert storage_stats.fields_saved == [
                ('for_runners', 'gpxmodel', 'track_svg'),
                ('for_runners', 'gpxmodel', 'gpx_file')
            ]
            assert storage_stats.fields_read == []

            output = buff.get_output()
            print(output)

            self.assertIn("Add new gpx tracks for user: normal_test_user", output)
            self.assertIn("1 - Add new track: 2018-02-21", output)
            self.assertIn("2 - Add new track: 2011-01-13", output)
            self.assertIn("Added 2 new gpx tracks.", output)
            self.assertIn("User normal_test_user has now 2 tracks.", output)

            qs = GpxModel.objects.filter(tracked_by__username=test_username)

            existing_tracks = [str(track) for track in qs]
            assert_pformat_equal(
                existing_tracks, ['2018-02-21', '2011-01-13']
            )

            assert qs.count() == 2
            assert storage_stats.fields_read == []



# Generated by Django 2.0.7 on 2018-07-19 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("for_runners", "0003_auto_20180712_2043")]

    operations = [
        migrations.AlterField(
            model_name="costmodel",
            name="name",
            field=models.CharField(help_text="The name of the item you dis pay for", max_length=15),
        )
    ]



# Generated by Django 2.0.7 on 2018-07-12 20:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("for_runners", "0002_auto_20180710_1910")]

    operations = [
        migrations.AlterModelOptions(
            name="participationmodel",
            options={
                "ordering": ("-event__start_date", "person"),
                "verbose_name": "Event Participation",
                "verbose_name_plural": "Event Participations",
            },
        ),
        migrations.RemoveField(model_name="gpxmodel", name="event"),
        migrations.RemoveField(model_name="participationmodel", name="distance"),
        migrations.AddField(
            model_name="gpxmodel",
            name="participation",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="tracks",
                to="for_runners.ParticipationModel",
            ),
        ),
        migrations.AddField(
            model_name="participationmodel",
            name="distance_km",
            field=models.DecimalField(
                decimal_places=4, default=0, help_text="Official track length in kilometer.", max_digits=7
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.1 on 2018-09-02 19:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("for_runners", "0005_auto_20180731_1855")]

    operations = [
        migrations.AlterField(
            model_name="gpxmodel",
            name="participation",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="track",
                to="for_runners.ParticipationModel",
            ),
        )
    ]



"""
    Just print version line on every call from commandline ;)
"""
import os
import sys

from django import __version__ as django_version

from for_runners import __version__


if __name__ == 'for_runners_project':
    if '--version' not in sys.argv:
        print(f'Django-ForRunners v{__version__} (Django v{django_version})', file=sys.stderr)
        print(f'DJANGO_SETTINGS_MODULE={os.environ["DJANGO_SETTINGS_MODULE"]!r}', file=sys.stderr)



"""
    created 17.11.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import datetime
import logging
import shutil
from pathlib import Path

from django.conf import settings

# https://github.com/jedie/django-for-runners
from for_runners.management.commands.base import BaseCommand
from for_runners.selectors.gpx import gpx_tracks, gpx_user_tracks, gpx_users
from for_runners.services.gpx_svg_generator import CsvGenerator


log = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    TODO: Use for_runners.admin.gpx_import_export.GpxModelResource
    """

    help = "Backup everything"

    def backup_database(self, *, backup_path):
        db_file = Path(settings.DATABASES["default"]["NAME"])
        if not db_file.is_file():
            print(f"Error database file not found: {db_file}")
        else:
            db_file_bak = Path(backup_path, db_file.name)
            print(f"Backup database file to: {db_file_bak}")
            shutil.copyfile(str(db_file), str(db_file_bak))

    def csv_user_export(self, *, backup_path):
        """
        Export all .gpx tracks
            * create sub directory from username
            * generate .csv file per user
        """
        for user in gpx_users():
            print(f"Export for user: {user.username}")

            out_path = Path(backup_path, user.username)
            out_path.mkdir(parents=True, exist_ok=False)

            user_csv_path = Path(backup_path, f"runnings_{user.username}.csv")

            tracks = 0

            with user_csv_path.open("w", encoding="utf-8", newline="") as csv_file:
                csv_generator = CsvGenerator(csv_file=csv_file, add_username=False)

                for track in gpx_user_tracks(user=user):
                    print(".", end="", flush=True)

                    filename = f"{track.get_short_slug()}.gpx"

                    file_path = Path(out_path, filename)
                    with file_path.open("w") as f:
                        f.write(track.gpx)

                    # output one csv row:
                    csv_generator.add_gpx_track(track=track)

                    tracks += 1

            print("\n%i trackes saved for user: %s\n" % (tracks, user.username))

    def csv_complete_export(self, *, backup_path):

        csv_path = Path(backup_path, "runnings.csv")
        print(f"Generate {csv_path}...")

        tracks = 0

        with csv_path.open("w", encoding="utf-8", newline="") as csv_file:
            csv_generator = CsvGenerator(csv_file=csv_file, add_username=True)

            for track in gpx_tracks(has_gpx=True):
                print(".", end="", flush=True)

                # output one csv row:
                csv_generator.add_gpx_track(track=track)

                tracks += 1

        print("\n%i trackes\n" % tracks)

    def handle(self, *args, **options):

        now = datetime.datetime.now()
        timestamp = now.strftime("%Y%m%d_%H%M")

        backup_path = Path(settings.PROJECT_PATH, "backups", timestamp)

        print("_" * 100)
        print(f" *** Create backup to: {backup_path} ***")
        print()

        backup_path.mkdir(parents=True, exist_ok=False)

        # Backup SQLite database file:
        self.backup_database(backup_path=backup_path)

        # Export gpx tracks and create user-csv-file:
        self.csv_user_export(backup_path=backup_path)

        # Generate .csv file for all tracks:
        self.csv_complete_export(backup_path=backup_path)

        # TODO: Save svg

        print("\n*** Backup completed.")



"""
    Base Django settings
"""

import logging
import sys as __sys
from pathlib import Path as __Path

from django.utils.translation import ugettext_lazy as _

from for_runners.app_settings import *  # noqa


###############################################################################

# Build paths relative to the project root:
PROJECT_PATH = __Path(__file__).parent.parent.parent
print(f'PROJECT_PATH:{PROJECT_PATH}', file=__sys.stderr)

if __Path('/.dockerenv').is_file():
    # We are inside a docker container
    BASE_PATH = __Path('/django_volumes')
    assert BASE_PATH.is_dir()
else:
    # Build paths relative to the current working directory:
    BASE_PATH = __Path().cwd().resolve()

print(f'BASE_PATH:{BASE_PATH}', file=__sys.stderr)

# Paths with Django dev. server:
# BASE_PATH...: .../django-for-runners
# PROJECT_PATH: .../django-for-runners/src
#
# Paths in Docker container:
# BASE_PATH...: /for_runners_volumes
# PROJECT_PATH: /usr/local/lib/python3.9/site-packages

###############################################################################


# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

# Serve static/media files by Django?
# In production Caddy should serve this!
SERVE_FILES = False


# SECURITY WARNING: keep the secret key used in production secret!
__SECRET_FILE = __Path(BASE_PATH, 'secret.txt').resolve()
if not __SECRET_FILE.is_file():
    print(f'Generate {__SECRET_FILE}')
    from secrets import token_urlsafe as __token_urlsafe
    __SECRET_FILE.open('w').write(__token_urlsafe(128))

SECRET_KEY = __SECRET_FILE.open('r').read().strip()


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',

    'bx_py_utils',  # https://github.com/boxine/bx_py_utils
    'import_export',  # https://github.com/django-import-export/django-import-export
    'dbbackup',  # https://github.com/django-dbbackup/django-dbbackup

    'axes',  # https://github.com/jazzband/django-axes
    'django_processinfo',  # https://github.com/jedie/django-processinfo/

    # Django-ForRunners
    'for_runners.apps.ForRunnersConfig',
    'for_runners_project.for_runners_helper_app',
]

ROOT_URLCONF = 'for_runners_project.urls'
WSGI_APPLICATION = 'for_runners_project.wsgi.application'
SITE_ID = 1

AUTHENTICATION_BACKENDS = [
    'axes.backends.AxesBackend',
    'django.contrib.auth.backends.ModelBackend',
]

MIDDLEWARE = [
    'django_processinfo.middlewares.ProcessInfoMiddleware',

    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',

    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',

    'axes.middleware.AxesMiddleware',  # AxesMiddleware should be the last middleware
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        "DIRS": [str(__Path(PROJECT_PATH, 'for_runners_project', 'templates'))],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.template.context_processors.media',
                'django.template.context_processors.csrf',
                'django.template.context_processors.tz',
                'django.template.context_processors.static',

                'for_runners.context_processors.for_runners_version_string',
            ],
        },
    },
]

# _____________________________________________________________________________
# Internationalization

LANGUAGE_CODE = 'en'

LANGUAGES = [
    ('de', _('German')),
    ('en', _('English')),
]
USE_I18N = True
USE_L10N = True
TIME_ZONE = 'Europe/Paris'
USE_TZ = True

# _____________________________________________________________________________
# Static files (CSS, JavaScript, Images)

STATIC_URL = '/static/'
STATIC_ROOT = str(__Path(BASE_PATH, 'static'))

MEDIA_URL = '/media/'
MEDIA_ROOT = str(__Path(BASE_PATH, 'media'))

# _____________________________________________________________________________
# Cache Backend

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake',
    }
}

# _____________________________________________________________________________
# django-processinfo

from django_processinfo import app_settings as PROCESSINFO  # noqa


PROCESSINFO.ADD_INFO = False  # Don't add info in HTML page

# _____________________________________________________________________________
# Django-dbbackup

DBBACKUP_STORAGE = 'django.core.files.storage.FileSystemStorage'
DBBACKUP_STORAGE_OPTIONS = {'location': str(__Path(BASE_PATH, 'backups'))}


# _____________________________________________________________________________
# cut 'pathname' in log output

old_factory = logging.getLogRecordFactory()


def cut_path(pathname, max_length):
    if len(pathname) <= max_length:
        return pathname
    return f'...{pathname[-(max_length - 3):]}'


def record_factory(*args, **kwargs):
    record = old_factory(*args, **kwargs)
    record.cut_path = cut_path(record.pathname, 30)
    return record


logging.setLogRecordFactory(record_factory)

# -----------------------------------------------------------------------------

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'colored': {  # https://github.com/borntyping/python-colorlog
            '()': 'colorlog.ColoredFormatter',
            'format': '%(log_color)s%(asctime)s %(levelname)8s %(cut_path)s:%(lineno)-3s %(message)s',
        }
    },
    'handlers': {'console': {'class': 'colorlog.StreamHandler', 'formatter': 'colored'}},
    'loggers': {
        '': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'django': {'handlers': ['console'], 'level': 'INFO', 'propagate': False},
        'axes': {'handlers': ['console'], 'level': 'WARNING', 'propagate': False},
        'matplotlib': {'handlers': ['console'], 'level': 'INFO', 'propagate': False},
        'django_tools': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'for_runners': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
    },
}



"""
    created 31.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018-2020 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import collections
import logging

from django.core.cache import cache
from geopy.geocoders import Nominatim


log = logging.getLogger(__name__)

WGS84_NDIGITS = 5

Address = collections.namedtuple("Address", ("short, full"))


def construct_short_address(address):
    log.debug(f'address={address}')
    parts = []

    if "village" in address and "county" in address:
        parts.append(f"{address['village']},")
        parts.append(address["county"])
    else:
        parts.append(
            address.get("city") or address.get("town")
            or address.get("county") or address.get("state")
        )
        parts.append(address.get("suburb"))

    results = []
    for part in parts:
        if part and part not in results:
            results.append(part)

    short_address = " ".join(results)
    return short_address


def reverse_geo(lat, lon):
    """
    Create short+full address string from given geo coordinates

    :return: Address named tuple
        short : string
            The "sort" Address
        full : string
            The "full" Address
    """
    lat2 = round(lat, ndigits=WGS84_NDIGITS)
    if lat2 != lat:
        log.debug(f'round lat={lat} -> {lat2}')

    lon2 = round(lon, ndigits=WGS84_NDIGITS)
    if lon2 != lon:
        log.debug(f'round lat={lon} -> {lon2}')

    log.debug(f'reverse_geo lat={lat2} lon={lon2}')

    cache_key = f'reverse_geo_{lat2}_{lon2}'
    log.debug('reverse geo cache key: %r', cache_key)
    address = cache.get(cache_key)
    if address:
        log.debug('reverse geo from cache')
        full_address, raw_address = address
    else:
        geolocator = Nominatim(user_agent="django-for-runners")

        # https://nominatim.org/release-docs/develop/api/Reverse/
        location = geolocator.reverse(
            query=f"{lat2}, {lon2}",
            # TODO: language={user language}
            zoom=17  # major and minor streets
        )
        full_address = location.address
        raw_address = location.raw["address"]
        address = (full_address, raw_address)
        log.debug('Store to cache: %r', cache_key)
        cache.set(
            cache_key, address,
            timeout=None  # cache forever
        )

    short_address = construct_short_address(address=raw_address)
    log.info(f'short_address={short_address}')

    return Address(short_address, full_address)


if __name__ == "__main__":
    print(reverse_geo("51.6239133", "6.9749074"))  # Bottrop Feldhausen



"""
    API connection to metaweather.com

    https://www.metaweather.com/api/#locationday
    https://www.metaweather.com/api/location/search/?lattlong=36.96,-122.02
    https://www.metaweather.com/api/location/2487956/2013/4/30/

    created 21.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import collections
import datetime
import logging
import statistics
import time

import requests
from requests import HTTPError


log = logging.getLogger(__name__)


class NoWeatherData(ValueError):
    pass


def request_json(url, timeout=3, user_agent="django-for-runners"):
    headers = {'user-agent': user_agent}

    start_time = time.time()
    try:
        r = requests.get(url, headers=headers, timeout=timeout)
        r.raise_for_status()
        data = r.json()
    except (HTTPError, ValueError) as err:
        print(f"ERROR: {err} (url: {url!r})")
        raise NoWeatherData
    else:
        response_time_ms = round((time.time() - start_time) * 1000, 1)
        print(f"Response in: {response_time_ms:.1f}ms (url: {url!r})")
        return data


class MetaWeatherCom:

    API_URL = "https://www.metaweather.com/api"

    def _request(self, url):
        url = self.API_URL + url
        data = request_json(url)
        return data

    _lat_lon2woeid_cache = {}

    def lat_lon2woeid(self, lat, lon, decimal_places=2):
        """
        lat/lon coordinates to WOEID (Where On Earth ID)

        :return: nearest woeid, json_data
        """
        lat = round(lat, decimal_places)
        lon = round(lon, decimal_places)

        try:
            result = self._lat_lon2woeid_cache[(lat, lon)]
            log.info("Fetch WOEID from cache!")
            return result
        except KeyError:
            log.info("Request WOEID")

        json_data = self._request(url=f"/location/search/?lattlong={lat},{lon}")
        # pprint(json_data)
        # [{'distance': 1836,
        #   'latt_long': '36.974018,-122.030952',
        #   'location_type': 'City',
        #   'title': 'Santa Cruz',
        #   'woeid': 2488853},
        #  {'distance': 43722,
        #   'latt_long': '37.338581,-121.885567',
        #   'location_type': 'City',
        #   'title': 'San Jose',
        #   'woeid': 2488042},
        #  {'distance': 49177,
        #   'latt_long': '37.39999,-122.079552',
        #   'location_type': 'City',
        #   'title': 'Mountain View',
        #   'woeid': 2455920},
        #  ...]
        last_distance = None
        nearest_woeid = None
        woe_data = None
        for result in json_data:
            distance = int(result["distance"])
            if last_distance is None or distance < last_distance:
                last_distance = distance
                woe_data = result
                nearest_woeid = int(woe_data["woeid"])

        result = (nearest_woeid, woe_data, json_data)
        self._lat_lon2woeid_cache[(lat, lon)] = result
        return result

    _location_day_cache = {}

    def location_day(self, woeid, date, max_seconds=60):

        url = f"/location/{woeid}/{date.year}/{date.month}/{date.day}/"
        try:
            json_data = self._location_day_cache[url]
        except KeyError:
            log.debug("Request %s...", url)
            json_data = self._request(url)
            self._location_day_cache[url] = json_data
        else:
            log.debug("Fetch %s from cache!", url)

        if not json_data:
            log.error("json response is empty")
            raise NoWeatherData("json response is empty")

        # pprint(json_data)
        # [{'air_pressure': 1005.36,
        # 'applicable_date': '2017-04-30',
        # 'created': '2017-05-01T07:19:02.690650Z',
        # 'humidity': 55,
        # 'id': 6301075443286016,
        # 'max_temp': 23.453333333333333,
        # 'min_temp': 10.014999999999999,
        # 'predictability': 70,
        # 'the_temp': 23.97666666666667,
        # 'visibility': 12.81857701026008,
        # 'weather_state_abbr': 'lc',
        # 'weather_state_name': 'Light Cloud',
        # 'wind_direction': 305.4904669344955,
        # 'wind_direction_compass': 'NW',
        # 'wind_speed': 5.540785988894192},
        # ...]
        data = collections.defaultdict(list)
        for result in json_data:
            created_string = result["created"]  # e.g.: 2018-06-20T22:30:29.227960Z
            created_datetime = datetime.datetime.strptime(created_string, "%Y-%m-%dT%H:%M:%S.%fZ")
            created_datetime = created_datetime.replace(tzinfo=date.tzinfo)

            delta = abs(date - created_datetime)
            data[delta.total_seconds()] = result

        temperatures = []
        weather_state_counter = collections.Counter()
        for delta_seconds, result in sorted(data.items()):
            if len(temperatures) > 0 and delta_seconds > max_seconds:
                # Skip if "created" date time is more than max_seconds, but collect at least one result
                continue

            temperature = statistics.median([result["min_temp"], result["the_temp"], result["max_temp"]])
            log.debug("Collect %.1f°C from %s (delta: %isec)", temperature, result["created"], delta_seconds)
            temperatures.append(temperature)

            weather_state_counter[result["weather_state_name"]] += 1

            # print("%s°C" % round(temperature), created_datetime, delta, delta_hours)

        temperature = statistics.median(temperatures)

        # print(weather_state_counter) # e.g.: Counter({'Light Cloud': 5, 'Showers': 2, 'Heavy Cloud': 1})
        weather_states = [
            item[0] for item in weather_state_counter.most_common()
        ]  # e.g.: ['Light Cloud', 'Showers', 'Heavy Cloud']
        # print(weather_states)
        weather_state = "/".join(weather_states[:2])  # e.g.: Light Cloud/Showers

        log.info("Result: %.1f°C %r", temperature, weather_state)

        return temperature, weather_state

        #
        # print(
        #     created_datetime,
        #     "%s°C" % round(result["min_temp"]),
        #     "%s°C" % round(result["the_temp"]),
        #     "%s°C" % round(result["max_temp"]),
        #     "-> %s°C" % round(temp),
        #     result["predictability"],
        #     result["weather_state_name"],
        # )

    def coordinates2weather(self, lat, lon, date, max_seconds=12):
        if date is None:
            raise NoWeatherData("Can't get weather if date is None!")

        nearest_woeid, woe_data, json_data = self.lat_lon2woeid(lat, lon)
        log.info("Use nearest WOEID: %i (%r)", nearest_woeid, woe_data)
        temperature, weather_state = self.location_day(
            woeid=nearest_woeid,
            date=date,
            max_seconds=max_seconds
        )
        return temperature, weather_state


meta_weather_com = MetaWeatherCom()

if __name__ == "__main__":
    # Duisburg:
    # https://www.metaweather.com/api/location/search/?lattlong=51.4109,6.7828
    # woeid=648820 # Essen, city

    # nearest_woeid, woe_data, json_data = MetaWeatherCom().lat_lon2woeid(36.96, -122.02)
    # print("woeid:", nearest_woeid, woe_data)
    # woeid: 2488853 {'distance': 1836, 'title': 'Santa Cruz', 'location_type': 'City',
    # 'woeid': 2488853, 'latt_long': '36.974018,-122.030952'}

    # /api/location/2487956/2013/4/30/ - San Francisco on 30th April 2013
    # MetaWeatherCom().location_day(woeid=2488853, date=datetime.date(year=2017, month=4, day=30))

    # Essen City on 21.06.2018
    # https://www.metaweather.com/de/648820/2018/6/20/
    temperature, weather_state = meta_weather_com.coordinates2weather(
        51.4109, 6.7828, date=datetime.datetime(year=2018, month=6, day=20, hour=20, minute=30)
    )
    print(temperature, weather_state)

    temperature, weather_state = meta_weather_com.coordinates2weather(
        51.4109, 6.7828, date=datetime.datetime(year=2018, month=6, day=20, hour=13, minute=30)
    )
    print(temperature, weather_state)

    temperature, weather_state = meta_weather_com.coordinates2weather(
        51.4109, 6.7828, date=datetime.datetime(year=2018, month=6, day=20, hour=4, minute=30)
    )
    print(temperature, weather_state)

    temperature, weather_state = meta_weather_com.coordinates2weather(
        51.4109, 6.7828, date=datetime.datetime(year=2018, month=6, day=20, hour=20, minute=30)
    )
    print(temperature, weather_state)



from unittest import TestCase

import requests_mock
from django.conf import settings
from django.core.cache import cache

from for_runners.geo import reverse_geo
from for_runners.tests.fixture_files import fixture_content
from for_runners.tests.utils import ClearCacheMixin


class GeoTests(ClearCacheMixin, TestCase):
    def test_reverse_geo(self):

        # first point from: for_runners/tests/fixture_files/garmin_connect_1.gpx
        lat = 51.43788929097354412078857421875
        lon = 6.617012657225131988525390625

        with requests_mock.mock() as m:
            m.get(
                (
                    'https://nominatim.openstreetmap.org/reverse'
                    '?lat=51.43789&lon=6.61701&format=json&addressdetails=1&zoom=17'
                ),
                headers={'Content-Type': 'application/json'},
                content=fixture_content('metaweather_5143789_661701.json')
            )
            address = reverse_geo(lat, lon)

        assert address.short == 'Moers'
        assert address.full == (
            '148, Filder Straße, Vinn, Moers, Kreis Wesel, Nordrhein-Westfalen, 47447, Deutschland'
        )

        # Check if cache works in tests:
        assert settings.CACHES['default']['BACKEND'] == (
            'django.core.cache.backends.locmem.LocMemCache'
        )
        cache.set('foo', 'bar', timeout=None)
        assert cache.get('foo') == 'bar'

        # Cache filled?
        address = cache.get('reverse_geo_51.43789_6.61701')
        assert address is not None
        assert address == (
            '148, Filder Straße, Vinn, Moers, Kreis Wesel, Nordrhein-Westfalen, 47447, Deutschland',
            {'city': 'Moers',
             'country': 'Deutschland',
             'country_code': 'de',
             'county': 'Kreis Wesel',
             'hamlet': 'Vinn',
             'house_number': '148',
             'postcode': '47447',
             'road': 'Filder Straße',
             'state': 'Nordrhein-Westfalen',
             'suburb': 'Moers'}
        )

        # Second request is cached -> no request
        with requests_mock.mock():
            address = reverse_geo(lat, lon)

        assert address.short == 'Moers'
        assert address.full == (
            '148, Filder Straße, Vinn, Moers, Kreis Wesel, Nordrhein-Westfalen, 47447, Deutschland'
        )



from pathlib import Path

from creole.setup_utils import update_rst_readme
from poetry_publish.publish import poetry_publish
from poetry_publish.utils.subprocess_utils import verbose_check_call

# https://github.com/jedie/django-for-runners
import for_runners


PACKAGE_ROOT = Path(for_runners.__file__).parent.parent.parent


def update_readme():
    return update_rst_readme(
        package_root=PACKAGE_ROOT,
        filename='README.creole'
    )


def publish():
    """
        Publish to PyPi
        Call this via:
            $ poetry run publish
    """
    verbose_check_call('make', 'pytest')  # don't publish if tests fail
    verbose_check_call('make', 'fix-code-style')  # don't publish if code style wrong

    poetry_publish(
        package_root=PACKAGE_ROOT,
        version=for_runners.__version__,
        creole_readme=True  # don't publish if README.rst is not up-to-date
    )


if __name__ == '__main__':
    update_readme()



"""
    created 17.11.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018-2019 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import logging

# https://github.com/jedie/django-for-runners
from for_runners.management.commands.base import BaseCommand
from for_runners.models import GpxModel
from for_runners.services.gpx_save_gpx import save_gpx_file
from for_runners.services.gpx_svg_generator import generate_svg


log = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    call this e.g.:

        ~/Django-ForRunners/bin$ ./manage recreate_files
    """

    help = "Recreate all svg, gpx files for existing gpx tracks on disk"

    def handle(self, *args, **options):
        qs = GpxModel.objects.all()
        total_count = qs.count()
        for no, gpx_track in enumerate(qs):
            print("[%i/%i] Generate for: %s" % (no, total_count, gpx_track))

            if not gpx_track.gpx:
                continue

            generate_svg(gpx_track=gpx_track, force=True)
            save_gpx_file(gpx_track=gpx_track, force=True)

            gpx_track.save()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['for_runners',
 'for_runners.admin',
 'for_runners.gpx_tools',
 'for_runners.management',
 'for_runners.management.commands',
 'for_runners.managers',
 'for_runners.migrations',
 'for_runners.models',
 'for_runners.selectors',
 'for_runners.services',
 'for_runners.signal_handlers',
 'for_runners.tests',
 'for_runners.tests.fixture_files',
 'for_runners.views',
 'for_runners_project',
 'for_runners_project.for_runners_helper_app',
 'for_runners_project.for_runners_helper_app.fixtures',
 'for_runners_project.for_runners_helper_app.management',
 'for_runners_project.for_runners_helper_app.management.commands',
 'for_runners_project.settings',
 'for_runners_project.tests']

package_data = \
{'': ['*'],
 'for_runners': ['static/*',
                 'static/chartjs/*',
                 'static/dygraphs/*',
                 'static/leaflet/*',
                 'static/leaflet/images/*',
                 'templates/admin/for_runners/*',
                 'templates/admin/for_runners/eventmodel/*',
                 'templates/admin/for_runners/gpxmodel/*',
                 'templates/admin/for_runners/gpxmodel/include/*',
                 'templates/for_runners/*',
                 'templates/for_runners/includes/*'],
 'for_runners_project': ['templates/*', 'templates/admin/*']}

install_requires = \
['autotask',
 'bx_django_utils',
 'bx_py_utils',
 'click',
 'colorlog',
 'django-axes',
 'django-dbbackup',
 'django-debug-toolbar',
 'django-import-export',
 'django-processinfo',
 'django-tools',
 'django>=2.2.0,<2.3.0',
 'geopy',
 'gpxpy',
 'gunicorn',
 'lxml',
 'matplotlib',
 'requests',
 'svgwrite']

extras_require = \
{'docker': ['docker-compose'], 'postgres': ['psycopg2-binary']}

entry_points = \
{'console_scripts': ['manage = for_runners_project.manage:main',
                     'publish = for_runners_project.publish:publish',
                     'update_rst_readme = '
                     'for_runners_project.publish:update_readme']}

setup_kwargs = {
    'name': 'django-for-runners',
    'version': '0.12.1',
    'description': 'Store your GPX tracks of your running (or other sports activity) in django.',
    'long_description': '=================\nDjango-ForRunners\n=================\n\n|Logo| Store your GPX tracks of your running (or other sports activity) in django.\n\n.. |Logo| image:: https://raw.githubusercontent.com/jedie/django-for-runners/master/src/for_runners/static/Django-ForRunners128.png\n\n+-----------------------------------+------------------------------------------------+\n| |Build Status on github|          | `github.com/jedie/django-for-runners/actions`_ |\n+-----------------------------------+------------------------------------------------+\n| |Coverage Status on codecov.io|   | `codecov.io/gh/jedie/django-for-runners`_      |\n+-----------------------------------+------------------------------------------------+\n| |Coverage Status on coveralls.io| | `coveralls.io/r/jedie/django-for-runners`_     |\n+-----------------------------------+------------------------------------------------+\n\n.. |Build Status on github| image:: https://github.com/jedie/django-for-runners/workflows/test/badge.svg?branch=master\n.. _github.com/jedie/django-for-runners/actions: https://github.com/jedie/django-for-runners/actions\n.. |Coverage Status on codecov.io| image:: https://codecov.io/gh/jedie/django-for-runners/branch/master/graph/badge.svg\n.. _codecov.io/gh/jedie/django-for-runners: https://codecov.io/gh/jedie/django-for-runners\n.. |Coverage Status on coveralls.io| image:: https://coveralls.io/repos/jedie/django-for-runners/badge.svg\n.. _coveralls.io/r/jedie/django-for-runners: https://coveralls.io/r/jedie/django-for-runners\n\n(The name **Django-ForRunners** has the origin from the great Android tracking app **ForRunners** by Benoît Hervier: `http://rvier.fr/#forrunners <http://rvier.fr/#forrunners>`_ )\n\n---------\nFeatures:\n---------\n\n* GPX track management:\n\n    * Upload GPX tracks\n\n    * Import GPX tracks from commandline\n\n    * Track analysis:\n\n        * basics: Track length / Duration / Pace / Hart rate Up-/Downhill\n\n        * Display route on OpenStreetMap map\n\n        * Graphs with elevation / heart rate / cadence (if available in GPX data)\n\n    * Data that is automatically extracted from the web:\n\n        * Start/finish Address from OpenStreetMap\n\n        * Start/finish weather information from metaweather.com\n\n    * Store additional data:\n\n        * Ideal track distance (for easier grouping/filtering tracks)\n\n* sports competitions Management:\n\n    * Create a List of Sport Events\n\n        * Add participation to a event\n\n        * link GPX track with a event participation\n\n        * Store you event participation:\n\n            * official track length\n\n            * measured finisher time\n\n            * Number of participants who have finished in your discipline\n\n        * Add links to webpages relatet to this event\n\n        * Record costs (entry fee, T-shirt etc.)\n\n* common\n\n    * Multiple user support (However: no rights management and currently only suitable for a handful of users)\n\n-----------------\nProject structure\n-----------------\n\nThere are two main directories:\n\n+---------------------+---------------------------------------------------+\n| directory           | description                                       |\n+=====================+===================================================+\n| **`/src/`_**        | The main  Django-ForRunners source code           |\n+---------------------+---------------------------------------------------+\n| **`/deployment/`_** | deploy  Django-ForRunners for production use case |\n+---------------------+---------------------------------------------------+\n\n.. _/src/: https://github.com/jedie/django-for-runners/tree/master/src\n.. _/deployment/: https://github.com/jedie/django-for-runners/tree/master/deployment\n\n-------\ninstall\n-------\n\nThere exists two kind of installation/usage:\n\n* local development installation using poetry\n\n* production use with docker-compose\n\n* Install as `YunoHost <https://yunohost.org>`_ App via `django-for-runners_ynh <https://github.com/YunoHost-Apps/django-for-runners_ynh>`_\n\nThis README contains only the information about local development installation.\n\nRead `/deployment/README <https://github.com/jedie/django-for-runners/tree/master/deployment#readme>`_ for instruction to install  Django-ForRunners on a root server.\n\nprepare\n=======\n\n::\n\n    ~$ git clone https://github.com/jedie/django-for-runners.git\n    ~$ cd django-for-runners/\n    ~/django-for-runners$ make\n    _________________________________________________________________\n    DjangoForRunners - *dev* Makefile\n    \n    install-poetry         install or update poetry\n    install                install DjangoForRunners via poetry\n    manage-update          Collectstatic + makemigration + migrate\n    update                 update the sources and installation\n    lint                   Run code formatters and linter\n    fix-code-style         Fix code formatting\n    tox-listenvs           List all tox test environments\n    tox                    Run pytest via tox with all environments\n    tox-py36               Run pytest via tox with *python v3.6*\n    tox-py37               Run pytest via tox with *python v3.7*\n    tox-py38               Run pytest via tox with *python v3.8*\n    pytest                 Run pytest\n    update-rst-readme      update README.rst from README.creole\n    publish                Release new version to PyPi\n    run-dev-server         Run the django dev server in endless loop.\n    createsuperuser        Create super user\n    messages               Make and compile locales message files\n    dbbackup               Backup database\n    dbrestore              Restore a database backup\n    run-docker-dev-server  Start docker containers with current dev source code\n\nlocal development installation\n==============================\n\n::\n\n    # install or update Poetry:\n    ~/django-for-runners$ make install-poetry\n    \n    # install  Django-ForRunners via poetry:\n    ~/django-for-runners$ make install\n    ...\n    \n    # Collectstatic + makemigration + migrate:\n    ~/django-for-runners$ make manage-update\n    \n    # Create a django super user:\n    ~/django-for-runners$ ./manage.sh createsuperuser\n    \n    # start local dev. web server:\n    ~/django-for-runners$ make run-dev-server\n\nThe web page is available in Port 8000, e.g.: ``http://127.0.0.1:8000/``\n\nlocal docker dev run\n====================\n\nYou can run the deployment docker containers with current source code with:\n\n::\n\n    ~/django-for-runners$ make run-docker-dev-server\n\nJust hit Cntl-C to stop the containers\n\nThe web page is available on Port 80, e.g.: ``http://localhost/``\n\nimport GPX files\n================\n\ne.g.:\n\n::\n\n    ~/django-for-runners$ poetry run manage import_gpx --username <django_username> ~/backups/gpx_files\n\n**Note:** It is no problem to start **import_gpx** with the same GPX files: Duplicate entries are avoided. The start/finish (time/latitude/longitude) are compared.\n\nbackup\n======\n\nCreate a backup into ``.../backups/<timestamp>/`` e.g.:\n\n::\n\n    ~/django-for-runners$ poetry run for_runners backup\n\nThe backup does:\n\n* backup the database\n\n* export all GPX tracks\n\n* generate .csv files:\n\n* a complete file with all running tracks\n\n* one file for every user\n\nregenerate all SVG files\n========================\n\n::\n\n    ~/django-for-runners$ poetry run for_runners recreate-svg\n\n-----------\nScreenshots\n-----------\n\n(All screenshots are here: `github.com/jedie/jedie.github.io/tree/master/screenshots/django-for-runners <https://github.com/jedie/jedie.github.io/tree/master/screenshots/django-for-runners>`_)\n\n------------------------------------------\nfor-runers v0.6.0 2018-07-31 GPX Track.png\n------------------------------------------\n\n|for-runers v0.6.0 2018-07-31 GPX Track.png|\n\n.. |for-runers v0.6.0 2018-07-31 GPX Track.png| image:: https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/django-for-runners/for-runers v0.6.0 2018-07-31 GPX Track.png\n\n-----------------------------------------\nfor-runners v0.4.0 2018-6-26 GPX info.png\n-----------------------------------------\n\n|for-runners v0.4.0 2018-6-26 GPX info.png|\n\n.. |for-runners v0.4.0 2018-6-26 GPX info.png| image:: https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/django-for-runners/for-runners v0.4.0 2018-6-26 GPX info.png\n\n---------------------------------------------\nfor-runners v0.6.0 2018-07-19 Event Costs.png\n---------------------------------------------\n\n|for-runners v0.6.0 2018-07-19 Event Costs.png|\n\n.. |for-runners v0.6.0 2018-07-19 Event Costs.png| image:: https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/django-for-runners/for-runners v0.6.0 2018-07-19 Event Costs.png\n\n----------------------\nprint a small overview\n----------------------\n\n|for-runners v0.10.0 2010-06-26 print small overview 1.png|\n\n.. |for-runners v0.10.0 2010-06-26 print small overview 1.png| image:: https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/django-for-runners/for-runners v0.10.0 2010-06-26 print small overview 1.png\n\n|for-runners v0.10.0 2010-06-26 print small overview 2.png|\n\n.. |for-runners v0.10.0 2010-06-26 print small overview 2.png| image:: https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/django-for-runners/for-runners v0.10.0 2010-06-26 print small overview 2.png\n\n---------\nrun tests\n---------\n\n::\n\n    ~/Django-ForRunners$ make test\n    \n    or:\n    \n    ~/Django-ForRunners$ make tox\n\n**Note:**\n\nTo run all tests, you need:\n\n* **Chromium Browser WebDriver** e.g.: ``apt install chromium-chromedriver``\n\n* **Firefox Browser WebDriver** aka **geckodriver**\n\ninstall **geckodriver** e.g.:\n\n::\n\n    ~$ cd /tmp\n    /tmp$ wget https://github.com/mozilla/geckodriver/releases/download/v0.20.1/geckodriver-v0.20.1-linux64.tar.gz -O geckodriver.tar.gz\n    /tmp$ sudo sh -c \'tar -x geckodriver -zf geckodriver.tar.gz -O > /usr/bin/geckodriver\'\n    /tmp$ sudo chmod +x /usr/bin/geckodriver\n    /tmp$ rm geckodriver.tar.gz\n    /tmp$ geckodriver --version\n    geckodriver 0.20.1\n    ...\n\n----------\nsome notes\n----------\n\nGPX storage\n===========\n\nCurrently we store the unchanged GPX data in a TextField.\n\nstatic files\n============\n\nWe collect some JavaScript files, for easier startup. These files are:\n\n+------------------+--------------------------------+---------------------------------+\n| Project Homepage | License                        | storage directory               |\n+==================+================================+=================================+\n| `leafletjs.com`_ | `Leaflet licensed under BSD`_  | `for_runners/static/leaflet/`_  |\n+------------------+--------------------------------+---------------------------------+\n| `dygraphs.com`_  | `dygraphs licensed under MIT`_ | `for_runners/static/dygraphs/`_ |\n+------------------+--------------------------------+---------------------------------+\n| `chartjs.org`_   | `Chart.js licensed under MIT`_ | `for_runners/static/chartjs/`_  |\n+------------------+--------------------------------+---------------------------------+\n\n.. _leafletjs.com: http://leafletjs.com\n.. _Leaflet licensed under BSD: https://github.com/Leaflet/Leaflet/blob/master/LICENSE\n.. _for_runners/static/leaflet/: https://github.com/jedie/django-for-runners/tree/master/for_runners/static/leaflet\n.. _dygraphs.com: http://dygraphs.com\n.. _dygraphs licensed under MIT: https://github.com/danvk/dygraphs/blob/master/LICENSE.txt\n.. _for_runners/static/dygraphs/: https://github.com/jedie/django-for-runners/tree/master/for_runners/static/dygraphs\n.. _chartjs.org: http://www.chartjs.org\n.. _Chart.js licensed under MIT: https://github.com/chartjs/Chart.js/blob/master/LICENSE.md\n.. _for_runners/static/chartjs/: https://github.com/jedie/django-for-runners/tree/master/for_runners/static/chartjs\n\nPrecision of coordinates\n========================\n\nGPX files from Garmin (can) contain:\n\n* latitude with 29 decimal places\n\n* longitude with 28 decimal places\n\n* elevation with 19 decimal places\n\nThe route on OpenStreetMap does not look more detailed, with more than 5 decimal places.\n\nSee also: `https://wiki.openstreetmap.org/wiki/Precision_of_coordinates <https://wiki.openstreetmap.org/wiki/Precision_of_coordinates>`_\n\n--------------------\nDjango compatibility\n--------------------\n\n+--------------------+----------------+---------------+\n| django-for-runners | django version | python        |\n+====================+================+===============+\n| >=v0.11.0          | 2.2.x LTS      | 3.7, 3.8, 3.9 |\n+--------------------+----------------+---------------+\n| >=v0.7.1           | 2.1            | 3.5, 3.6, 3.7 |\n+--------------------+----------------+---------------+\n| v0.5.x             | 2.0            | 3.5, 3.6, 3.7 |\n+--------------------+----------------+---------------+\n\n(See also combinations in `tox.ini <https://github.com/jedie/django-for-runners/blob/master/tox.ini>`_ and `github actions <https://github.com/jedie/django-for-runners/blob/master/.github/workflows/pythonapp.yml>`_)\n\n------------------------------\nBackwards-incompatible changes\n------------------------------\n\nOlder changes, see:\n\n`https://github.com/jedie/django-for-runners/blob/v0.10.1/README.creole#backwards-incompatible-changes <https://github.com/jedie/django-for-runners/blob/v0.10.1/README.creole#backwards-incompatible-changes>`_\n\nv0.12.0\n=======\n\nMove main project sources into "/src/"\nAdd deployment setup into "/development/"\n\n-------\nhistory\n-------\n\n* `compare v0.12.1...master <https://github.com/jedie/django-for-runners/compare/v0.12.1...master>`_ **dev** \n\n    * tbc\n\n* `22.11.2021 - v0.12.1 <https://github.com/jedie/django-for-runners/compare/v0.12.0...v0.12.1>`_:\n\n    * update requirements\n\n* `17.01.2021 - v0.12.0 <https://github.com/jedie/django-for-runners/compare/v0.11.0...v0.12.0>`_:\n\n    * refactor project structure and add a deployment via docker-compose setup\n\n    * installation as YunoHost app: `https://github.com/YunoHost-Apps/django-for-runners_ynh <https://github.com/YunoHost-Apps/django-for-runners_ynh>`_\n\n    * Add django-axes and django-processinfo\n\n    * Mock requests in tests\n\n* `04.07.2020 - v0.11.0 <https://github.com/jedie/django-for-runners/compare/v0.10.1...v0.11.0>`_:\n\n    * refactor gpx import code\n\n    * update tests\n\n    * Use poetry and add Makefile\n\n    * update code style\n\n* `09.08.2019 - v0.10.1 <https://github.com/jedie/django-for-runners/compare/v0.10.0...v0.10.1>`_:\n\n    * Enhance "Event Participation" admin view: Add start date and costs in table\n\n* `26.06.2019 - v0.10.0 <https://github.com/jedie/django-for-runners/compare/v0.9.0...v0.10.0>`_:\n\n    * NEW: GPX Admin action to print a small overview from one or a few tracks\n\n    * Accept optional server bind address, e.g.: ``for_runners run-server 127.0.0.1:8080``\n\n* `02.04.2019 - v0.9.0 <https://github.com/jedie/django-for-runners/compare/v0.8.1...v0.9.0>`_:\n\n    * NEW: Update complete environment installation with: ``for_runners update``\n\n    * Move the SQlite database to virtualenv root dir, e.g.: ``~/Django-ForRunners/Django-ForRunners-database.sqlite3``\n\n    * NEW: save every gpx track to disk\n\n    * NEW: Backup/export via cli: ``$ for_runners backup``\n\n    * NEW: export GPX Data via ``django-import-export``\n\n    * Create xdg-open desktop starter under linux here: ``~/Django-ForRunners/Django-ForRunners``\n\n    * refactor the startup process:\n\n        * auto loop the server process\n\n        * open web browser on first start\n\n        * enable autotask\n\n    * rename ``for_runners_test_project`` to ``for_runners_project``\n\n* `03.09.2018 - v0.8.1 <https://github.com/jedie/django-for-runners/compare/v0.8.0...v0.8.1>`_:\n\n    * Fix "try-out" section in README, again ;(\n\n* `03.09.2018 - v0.8.0 <https://github.com/jedie/django-for-runners/compare/v0.7.1...v0.8.0>`_:\n\n    * NEW: shell script for easier boot/install, see above\n\n* `02.09.2018 - v0.7.1 <https://github.com/jedie/django-for-runners/compare/v0.7.0...v0.7.1>`_:\n\n    * Update to Django 2.1\n\n    * Bugfix Tests\n\n* `02.09.2018 - v0.7.0 <https://github.com/jedie/django-for-runners/compare/v0.6.0...v0.7.0>`_:\n\n    * Use dygraphs in GPX Track change admin view\n\n    * Sync mouse over from Elevation/Headrate/Cadence dygraphs to leaflet open streep map\n\n    * Fix "try-out" section in README (`Thanks adsworth for reporting <https://github.com/jedie/django-for-runners/pull/1>`_)\n\n    * Add links from gpx tracks to other admin change view\n\n    * Bugfixes\n\n    * internals:\n\n        * refactor stuff around track duration/length\n\n        * move manipluation of list_display and list_filter `contributed by adsworth <https://github.com/jedie/django-for-runners/pull/2>`_\n\n* `19.07.2018 - v0.6.0 <https://github.com/jedie/django-for-runners/compare/v0.5.0...v0.6.0>`_:\n\n    * NEW: event participation\n\n    * NEW: costs of event participation (e.g.: entry fee for the competition, cost of a T-Shirt etc.)\n\n    * NEW: Display statistics of events/costs per user and total\n\n* `04.07.2018 - v0.5.0 <https://github.com/jedie/django-for-runners/compare/v0.4.0...v0.5.0>`_:\n\n    * remove Django-CMS\n\n    * update to Django v2.0\n\n    * NOTE: The migrations are simply replaced! So you must delete your database, e.g.: ``src/django-for-runners$ rm test_project_db.sqlite3``\n\n    * Add \'net duration\' field, for the officially measured time and use it for calculations if available.\n\n    * Create django manage command to fill some base data: ``$ ./manage.py fill_basedata``\n\n    * speedup by deactivating some django debug toolbar panels\n\n* `26.06.2018 - v0.4.0 <https://github.com/jedie/django-for-runners/compare/v0.3.0...v0.4.0>`_:\n\n    * combine track filters with statistic views\n\n    * NEW: GPX info (See length, point count and Average distance in meters between the points)\n\n    * NEW: Display GPX metadata\n\n    * Add \'creator\' to every track and use it as changelist filter\n\n    * remove Streetmap image generated via `geotiler <https://pypi.org/project/geotiler/>`_\n\n    * Speedup by using a cache for gpxpy instances\n\n* `23.06.2018 - v0.3.0 <https://github.com/jedie/django-for-runners/compare/v0.2.0...v0.3.0>`_:\n\n    * Start adding statistics (See screenshot above)\n\n    * add weather information from `metaweather.com <https://www.metaweather.com/>`_ to every track\n\n* `21.06.2018 - v0.2.0 <https://github.com/jedie/django-for-runners/compare/v0.1.1...v0.2.0>`_:\n\n    * Display elevations, heart_rates and cadence_values if available\n\n    * Add kilometer markers to OpenStreetMap\n\n* `15.06.2018 - v0.1.1 <https://github.com/jedie/django-for-runners/compare/v0.1.0...v0.1.1>`_:\n\n    * a few bugfixes\n\n* `15.06.2018 - v0.1.0 <https://github.com/jedie/django-for-runners/compare/v0.0.4...v0.1.0>`_:\n\n    * Render interactive OpenStreetMap track map with Leaflet JS\n\n* `12.06.2018 - v0.0.4 <https://github.com/jedie/django-for-runners/compare/v0.0.3...v0.0.4>`_:\n\n    * Better Events model\n\n    * GPX error handling\n\n    * more tests\n\n    * Bugfix for Python 3.5 (Geotiler needs Python 3.6 or later)\n\n* `12.06.2018 - v0.0.3 <https://github.com/jedie/django-for-runners/compare/v0.0.2...v0.0.3>`_:\n\n    * display min/average/max heart rate\n\n    * use autotask to generate the MAP in background\n\n* `31.05.2018 - v0.0.2 <https://github.com/jedie/django-for-runners/compare/v0.0.1...v0.0.2>`_:\n\n    * generate SVG \'icon\' from GPX track\n\n* v0.0.1 - 30.05.2018\n\n    * Just create a pre-alpha release to save the PyPi package name ;)\n\n-----\nlinks\n-----\n\n+----------+-------------------------------------------------+\n| Homepage | `http://github.com/jedie/django-for-runners`_   |\n+----------+-------------------------------------------------+\n| PyPi     | `https://pypi.org/project/django-for-runners/`_ |\n+----------+-------------------------------------------------+\n\n.. _http://github.com/jedie/django-for-runners: http://github.com/jedie/django-for-runners\n.. _https://pypi.org/project/django-for-runners/: https://pypi.org/project/django-for-runners/\n\nactivity exporter\n=================\n\nIt\'s sometimes hard to find a working project for exporting activities.\nSo here tools that i use currently:\n\n* `Garmin-Connect-Export <https://github.com/rsjrny/Garmin-Connect-Export>`_ from rsjrny\n\nalternatives (OpenSource only)\n==============================\n\n* `https://github.com/pytrainer/pytrainer <https://github.com/pytrainer/pytrainer>`_ (Desktop Program)\n\n* `https://github.com/GoldenCheetah/GoldenCheetah/ <https://github.com/GoldenCheetah/GoldenCheetah/>`_ (Desktop Program)\n\nOnline tools:\n\n* `https://www.j-berkemeier.de/ShowGPX.html <https://www.j-berkemeier.de/ShowGPX.html>`_ (de)\n\n-------\ncredits\n-------\n\nThe whole thing is based on many excellent projects. Especially the following:\n\n* `gpxpy <https://pypi.org/project/gpxpy/>`_ GPX file parser\n\n* `Leaflet JS <https://leafletjs.com>`_ A JS library for interactive maps used to render the track on `OpenStreetMap <https://openstreetmap.org/>`_\n\n* `dygraphs <http://dygraphs.com>`_ open source JavaScript charting library\n\n* `Chart.js <https://www.chartjs.org>`_ HTML5 Charts\n\n* `geopy <https://pypi.org/project/geopy/>`_ Get geo location names of the GPX track start/end point\n\n* `matplotlib <https://pypi.org/project/matplotlib/>`_ plotting 2D graphics\n\n* `autotask <https://pypi.org/project/autotask/>`_ schedule background jobs\n\n* `svgwrite <https://pypi.org/project/svgwrite/>`_ Generating SVG file\n\n--------\ndonation\n--------\n\n* `paypal.me/JensDiemer <https://www.paypal.me/JensDiemer>`_\n\n* `Flattr This! <https://flattr.com/submit/auto?uid=jedie&url=https%3A%2F%2Fgithub.com%2Fjedie%2Fdjango-for-runners%2F>`_\n\n* Send `Bitcoins <http://www.bitcoin.org/>`_ to `1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F <https://blockexplorer.com/address/1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F>`_\n\n------------\n\n``Note: this file is generated from README.creole 2021-11-22 23:13:24 with "python-creole"``',
    'author': 'JensDiemer',
    'author_email': 'git@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jedie/django-for-runners',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)



# Generated by Django 2.1.8 on 2019-04-02 16:39

from django.db import migrations


def forwards(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [("for_runners", "0008_auto_20190402_1731")]

    operations = [migrations.RunPython(forwards, migrations.RunPython.noop)]



"""
    created 21.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
from for_runners.tests.base import BaseTestCase


class TestBase(BaseTestCase):
    def test_assert_equal_rounded(self):
        self.assert_equal_rounded(1.21, 1.22, decimal_places=1)
        self.assert_equal_rounded([1.57, 1.6], [1.6, 1.58], decimal_places=1)
        self.assert_equal_rounded((1.57, 1.6), (1.6, 1.58), decimal_places=1)
        with self.assertRaises(AssertionError):
            self.assert_equal_rounded(0.12345, 0.12346)



"""
    ./manage.py load_dynamic_fixtures
"""
import logging

from django.conf import settings
from django.utils import translation
# https://github.com/Peter-Slump/django-dynamic-fixtures
from dynamic_fixtures.fixtures import BaseFixture

from for_runners.models import DisciplineModel


log = logging.getLogger(__name__)


class Fixture(BaseFixture):
    def load(self):
        print("_" * 79)
        print("Create DisciplineModel examples:")

        language_code = settings.LANGUAGE_CODE
        with translation.override(language_code):
            DisciplineModel.objects.get_or_create(name="Running")



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018-2020 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

__version__ = "0.12.1"

# https://docs.djangoproject.com/en/2.0/ref/applications/#configuring-applications-ref
default_app_config = "for_runners.apps.ForRunnersConfig"



import logging

from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.views.generic.base import View
from django.views.static import serve


logger = logging.getLogger(__name__)


class UserMediaView(View):
    """
    Serve MEDIA_URL files, but check the current user:
    """

    def get(self, request, user_name, path):
        # TODO: Change from user name to ID?
        media_path = f'{user_name}/{path}'

        logger.debug('Serve: %r', media_path)

        if not request.user.is_superuser:
            if request.user.username != user_name:
                # A user tries to access a file from a other use?
                if request.user.id is None:
                    logger.error(f'Anonymous try to access files from: {user_name!r}')
                else:
                    logger.error(f'Wrong user ID: {request.user.id!r} is not {user_name!r}')
                raise PermissionDenied

        # Send the file to the user:
        return serve(
            request,
            path=media_path,
            document_root=settings.MEDIA_ROOT,
            show_indexes=False
        )



from django.conf import settings
from django.conf.urls import include, static, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.urls import path
from django.views.generic import RedirectView

from for_runners.views.media_files import UserMediaView


admin.autodiscover()


urlpatterns = i18n_patterns(
    path("admin/", admin.site.urls),

    # until there is not real CMS pages: redirect to the interesting admin page:
    url(r"^$", RedirectView.as_view(url="/admin/for_runners/gpxmodel/")),
)


urlpatterns = [
    # TODO: Change from user name to ID?
    path('media/<slug:user_name>/<path:path>', UserMediaView.as_view()),
] + urlpatterns


if settings.SERVE_FILES:
    urlpatterns += static.static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)


if settings.DEBUG:
    import debug_toolbar
    urlpatterns = [url(r'^__debug__/', include(debug_toolbar.urls))] + urlpatterns



"""
    created 17.11.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import logging

from django.contrib.auth import get_user_model
from django.db.models import Count

# https://github.com/jedie/django-for-runners
from for_runners.models import GpxModel


log = logging.getLogger(__name__)


def gpx_users():
    User = get_user_model()
    qs = User.objects.annotate(num_tracks=Count("gpxmodel_createby"))
    qs = qs.filter(num_tracks__gt=0)
    return qs


def gpx_tracks(has_gpx=True):
    qs = GpxModel.objects.all()
    if has_gpx:
        qs = qs.exclude(gpx="")
    return qs


def gpx_user_tracks(user, has_gpx=True):
    qs = gpx_tracks(has_gpx=has_gpx)
    qs = qs.filter(tracked_by=user)
    return qs



from pathlib import Path

from django_tools.unittest_utils.assertments import assert_is_file


FIXTURES_PATH = Path(__file__).parent


def fixture_content(file_name, mode='rb'):
    file_path = FIXTURES_PATH / file_name
    assert_is_file(file_path)
    with file_path.open(mode) as f:
        return f.read()



# flake8: noqa: E405

"""
    Django settings for local development


"""

import sys as __sys

from for_runners_project.settings.base import *  # noqa


# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True


# Serve static/media files for local development:
SERVE_FILES = True


# Disable caches:
CACHES = {'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}

# Required for the debug toolbar to be displayed:
INTERNAL_IPS = ('127.0.0.1', '0.0.0.0', 'localhost')

ALLOWED_HOSTS = INTERNAL_IPS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': str(BASE_PATH / 'Django-ForRunners-database.sqlite3'),
        # https://docs.djangoproject.com/en/dev/ref/databases/#database-is-locked-errors
        'timeout': 30,
    }
}
print(f'Use Database: {DATABASES["default"]["NAME"]!r}', file=__sys.stderr)

# _____________________________________________________________________________

# Download map via geotiler in for_runners.gpx_tools.gpxpy2map.generate_map
MAP_DOWNLOAD = True

# _____________________________________________________________________________
# Django-Debug-Toolbar

INSTALLED_APPS += ['debug_toolbar']
MIDDLEWARE += ['debug_toolbar.middleware.DebugToolbarMiddleware']

DEBUG_TOOLBAR_PATCH_SETTINGS = True
from debug_toolbar.settings import CONFIG_DEFAULTS as DEBUG_TOOLBAR_CONFIG  # noqa


# Disable some more panels that will slow down the page:
DEBUG_TOOLBAR_CONFIG['DISABLE_PANELS'].add('debug_toolbar.panels.sql.SQLPanel')
DEBUG_TOOLBAR_CONFIG['DISABLE_PANELS'].add('debug_toolbar.panels.cache.CachePanel')

# don't load jquery from ajax.googleapis.com, just use django's version:
DEBUG_TOOLBAR_CONFIG['JQUERY_URL'] = STATIC_URL + 'admin/js/vendor/jquery/jquery.min.js'

DEBUG_TOOLBAR_CONFIG['SHOW_TEMPLATE_CONTEXT'] = True
DEBUG_TOOLBAR_CONFIG['SHOW_COLLAPSED'] = True  # Show toolbar collapsed by default.
DEBUG_TOOLBAR_CONFIG['SHOW_TOOLBAR_CALLBACK'] = 'for_runners_project.middlewares.djdt_show'
